'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var algosdk = require('algosdk');
var algosdk__default = _interopDefault(algosdk);
var _asyncToGenerator = _interopDefault(require('@babel/runtime/helpers/asyncToGenerator'));
var _regeneratorRuntime = _interopDefault(require('@babel/runtime/regenerator'));
var validate = _interopDefault(require('bitcoin-address-validation'));
var axios = _interopDefault(require('axios'));
var polling = _interopDefault(require('light-async-polling'));
var _toConsumableArray = _interopDefault(require('@babel/runtime/helpers/toConsumableArray'));
var BigNumber = _interopDefault(require('bignumber.js'));
var Web3PromiEvent = _interopDefault(require('web3-core-promievent'));
var eosjs = require('eosjs');
var eosjsJssig = require('eosjs/dist/eosjs-jssig');
var fetch = _interopDefault(require('node-fetch'));
var encoding = _interopDefault(require('text-encoding'));
var _defineProperty = _interopDefault(require('@babel/runtime/helpers/defineProperty'));
var msgpack = require('@msgpack/msgpack');

function isValidAddress(str) {
  var allowAppId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  if (allowAppId && parseInt(str)) return true;
  return algosdk__default.isValidAddress(str);
}
var waitForTransactionConfirmation = function waitForTransactionConfirmation(_client, _txIdx) {
  return algosdk__default.waitForConfirmation(_client, _txIdx, 4);
};

var algo = /*#__PURE__*/Object.freeze({
  __proto__: null,
  isValidAddress: isValidAddress,
  waitForTransactionConfirmation: waitForTransactionConfirmation
});

var Eosio = 'eosio';
var Ethereum = 'ethereum';
var Bitcoin = 'bitcoin';
var Litecoin = 'litecoin';
var Telos = 'telos';
var Dogecoin = 'dogecoin';
var BinanceSmartChain = 'binance-smart-chain';
var Polygon = 'polygon';
var Xdai = 'xdai';
var Ravencoin = 'ravencoin';
var Lbry = 'lbry';
var Ultra = 'ultra';
var Arbitrum = 'arbitrum';
var Luxochain = 'luxochain';
var Algorand = 'algorand';
var Fantom = 'fantom';
var Ore = 'ore';
var Libre = 'libre';
var Testnet = 'testnet';
var Mainnet = 'mainnet';
var BitcoinMainnet = 'mainnet';
var BitcoinTestnet = 'testnet';
var EthereumMainnet = 'mainnet';
var EthereumRopsten = 'testnet_ropsten';
var EosioMainnet = 'mainnet';
var EosioJungle3 = 'testnet_jungle3';
var LitecoinMainnet = 'mainnet';
var LitecoinTestnet = 'testnet';
var TelosMainnet = 'mainnet';
var DogecoinMainnet = 'mainnet';
var BinanceSmartChainMainnet = 'mainnet';
var PolygonMainnet = 'mainnet';
var XdaiMainnet = 'mainnet';
var RavencoinMainnet = 'mainnet';
var LbryMainnet = 'mainnet';
var UltraMainnet = 'mainnet';
var UltraTestnet = 'testnet';
var ArbitrumMainnet = 'mainnet';
var LuxochainMainnet = 'mainnet';
var AlgorandMainnet = 'mainnet';
var FantomMainnet = 'mainnet';
var OreMainnet = 'mainnet';
var LibreMainnet = 'mainnet';
var pBTC = 'pbtc';
var pLTC = 'pltc';
var pETH = 'peth';
var pWETH = 'pweth';
var pMKR = 'pmkr';
var pLINK = 'plink';
var pYFI = 'pyfi';
var PNT = 'pnt';
var PTERIA = 'pteria';
var pUNI = 'puni';
var pBAND = 'pband';
var pBAL = 'pbal';
var pCOMP = 'pcomp';
var pSNX = 'psnx';
var pOMG = 'pomg';
var pDAI = 'pdai';
var pANT = 'pant';
var pLRC = 'plrc';
var pUOS = 'puos';
var pBAT = 'pbat';
var pREP = 'prep';
var pZRX = 'pzrx';
var pPNK = 'ppnk';
var pDOGE = 'pdoge';
var pEOS = 'peos';
var IQ = 'iq';
var TLOS = 'tlos';
var pOPIUM = 'popium';
var pBCP = 'pbcp';
var pDEFIPlusPlus = 'pdefiplusplus';
var CGG = 'cgg';
var pUSDC = 'pusdc';
var pUSDT = 'pusdt';
var pRVN = 'prvn';
var pOPEN = 'popen';
var OCP = 'ocp';
var ANRX = 'anrx';
var TFF = 'tff';
var pSAFEMOON = 'psafemoon';
var EFX = 'efx';
var pSEEDS = 'pseeds';
var pLBC = 'plbc';
var USDO = 'usdo';
var GALA = 'gala';
var ZMT = 'zmt';
var BIST = 'bist';
var pVAI = 'pvai';
var WSB = 'wsb';
var LUXO = 'luxo';
var pTET = 'ptet';
var pKEYS = 'pkeys';
var OATH = 'oath';
var pFTM = 'pftm';
var pWFTM = 'pwftm';
var NUCO = 'nuco';
var ORE = 'ore';

var BLOCKSTREAM_BASE_TESTNET_ENDPOINT = 'https://blockstream.info/testnet/api/';
var BLOCKSTREAM_BASE_MAINNET_ENDPOINT = 'https://blockstream.info/api/';
var _getEsploraApi = function _getEsploraApi(_network) {
  return axios.create({
    baseURL: _network === Mainnet ? BLOCKSTREAM_BASE_MAINNET_ENDPOINT : BLOCKSTREAM_BASE_TESTNET_ENDPOINT,
    timeout: 50000,
    headers: {
      'Content-Type': 'text/plain'
    }
  });
};
var _makeEsploraApiCall = function _makeEsploraApiCall(_network, _callType, _apiPath, _params) {
  return new Promise(function (resolve, reject) {
    _getEsploraApi(_network)[_callType.toLowerCase()](_apiPath, _params).then(function (_res) {
      return resolve(_res.data);
    }).catch(function (_err) {
      return reject(_err);
    });
  });
};
var broadcastTransaction = function broadcastTransaction(_network, _tx) {
  return _makeEsploraApiCall(_network, 'POST', '/tx', _tx);
};
var getUtxoByAddress = function getUtxoByAddress(_network, _address) {
  return _makeEsploraApiCall(_network, 'GET', "/address/".concat(_address, "/utxo"));
};
var getTransactionHexById = function getTransactionHexById(_network, _txId) {
  return _makeEsploraApiCall(_network, 'GET', "/tx/".concat(_txId, "/hex"));
};
var isValidAddress$1 = function isValidAddress(_address) {
  return Boolean(validate(_address));
};
var monitorUtxoByAddress = function () {
  var _ref = _asyncToGenerator( _regeneratorRuntime.mark(function _callee2(_network, _address, _eventEmitter, _pollingTime, _broadcastEventName, _confirmationEventName) {
    var _maxRetries,
        isBroadcasted,
        txId,
        utxos,
        retries,
        _args2 = arguments;
    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _maxRetries = _args2.length > 6 && _args2[6] !== undefined ? _args2[6] : 5;
            isBroadcasted = false;
            txId = null;
            utxos = [];
            retries = 0;
            _context2.next = 7;
            return polling( _asyncToGenerator( _regeneratorRuntime.mark(function _callee() {
              return _regeneratorRuntime.wrap(function _callee$(_context) {
                while (1) {
                  switch (_context.prev = _context.next) {
                    case 0:
                      _context.prev = 0;
                      _context.next = 3;
                      return _makeEsploraApiCall(_network, 'GET', "/address/".concat(_address, "/utxo"));
                    case 3:
                      utxos = _context.sent;
                      if (!(utxos.length > 0)) {
                        _context.next = 20;
                        break;
                      }
                      if (!utxos[0].status.confirmed) {
                        _context.next = 13;
                        break;
                      }
                      if (!isBroadcasted) {
                        _eventEmitter.emit(_broadcastEventName, utxos[0]);
                        _eventEmitter.emit('onBtcTxBroadcasted', utxos[0]);
                      }
                      _eventEmitter.emit(_confirmationEventName, utxos[0]);
                      _eventEmitter.emit('onBtcTxConfirmed', utxos[0]);
                      txId = utxos[0].txid;
                      return _context.abrupt("return", true);
                    case 13:
                      if (isBroadcasted) {
                        _context.next = 18;
                        break;
                      }
                      isBroadcasted = true;
                      _eventEmitter.emit(_broadcastEventName, utxos[0]);
                      _eventEmitter.emit('onBtcTxBroadcasted', utxos[0]);
                      return _context.abrupt("return", false);
                    case 18:
                      _context.next = 21;
                      break;
                    case 20:
                      return _context.abrupt("return", false);
                    case 21:
                      _context.next = 29;
                      break;
                    case 23:
                      _context.prev = 23;
                      _context.t0 = _context["catch"](0);
                      if (!(retries === _maxRetries)) {
                        _context.next = 27;
                        break;
                      }
                      throw _context.t0;
                    case 27:
                      retries += 1;
                      return _context.abrupt("return", false);
                    case 29:
                    case "end":
                      return _context.stop();
                  }
                }
              }, _callee, null, [[0, 23]]);
            })), _pollingTime);
          case 7:
            return _context2.abrupt("return", txId);
          case 8:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return function monitorUtxoByAddress(_x, _x2, _x3, _x4, _x5, _x6) {
    return _ref.apply(this, arguments);
  };
}();
var waitForTransactionConfirmation$1 = function () {
  var _ref3 = _asyncToGenerator( _regeneratorRuntime.mark(function _callee4(_network, _tx, _pollingTime) {
    var transaction;
    return _regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            transaction = null;
            _context4.next = 3;
            return polling( _asyncToGenerator( _regeneratorRuntime.mark(function _callee3() {
              return _regeneratorRuntime.wrap(function _callee3$(_context3) {
                while (1) {
                  switch (_context3.prev = _context3.next) {
                    case 0:
                      _context3.prev = 0;
                      _context3.next = 3;
                      return _makeEsploraApiCall(_network, 'GET', "/tx/".concat(_tx));
                    case 3:
                      transaction = _context3.sent;
                      if (!(!transaction || !transaction.status)) {
                        _context3.next = 6;
                        break;
                      }
                      return _context3.abrupt("return", false);
                    case 6:
                      return _context3.abrupt("return", transaction.status.confirmed);
                    case 9:
                      _context3.prev = 9;
                      _context3.t0 = _context3["catch"](0);
                      return _context3.abrupt("return", false);
                    case 12:
                    case "end":
                      return _context3.stop();
                  }
                }
              }, _callee3, null, [[0, 9]]);
            })), _pollingTime);
          case 3:
            return _context4.abrupt("return", transaction);
          case 4:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));
  return function waitForTransactionConfirmation(_x7, _x8, _x9) {
    return _ref3.apply(this, arguments);
  };
}();

var btc = /*#__PURE__*/Object.freeze({
  __proto__: null,
  broadcastTransaction: broadcastTransaction,
  isValidAddress: isValidAddress$1,
  getUtxoByAddress: getUtxoByAddress,
  getTransactionHexById: getTransactionHexById,
  monitorUtxoByAddress: monitorUtxoByAddress,
  waitForTransactionConfirmation: waitForTransactionConfirmation$1
});

var DOGE_CHAIN_API = 'https://dogechain.info/api/v1';
var _getDogeChainApi = function _getDogeChainApi(_network) {
  if (_network === Testnet) throw new Error('Dogecoin on Testnet is not supported');
  return axios.create({
    baseURL: DOGE_CHAIN_API,
    timeout: 50000,
    headers: {
      'Content-Type': 'application/json'
    }
  });
};
var _makeDogeChainApiCall = function _makeDogeChainApiCall(_network, _callType, _apiPath, _params) {
  return new Promise(function (resolve, reject) {
    _getDogeChainApi(_network)[_callType.toLowerCase()](_apiPath, _params).then(function (_res) {
      return resolve(_res.data);
    }).catch(function (_err) {
      return reject(_err);
    });
  });
};
var broadcastTransaction$1 = function broadcastTransaction(_network, _tx) {
  return _makeDogeChainApiCall(_network, 'POST', '/pushtx', {
    tx: _tx
  });
};
var getUtxoByAddress$1 = function getUtxoByAddress(_network, _address) {
  return new Promise(function (_resolve, _reject) {
    return _makeDogeChainApiCall(_network, 'GET', "/unspent/".concat(_address, "/")).then(function (_ref) {
      var unspent_outputs = _ref.unspent_outputs,
          success = _ref.success,
          error = _ref.error;
      return success ? _resolve(unspent_outputs) : _reject(error);
    }).catch(_reject);
  });
};
var isValidAddress$2 = function isValidAddress(_address) {
  var res = _address.match(/D{1}[5-9A-HJ-NP-U]{1}[1-9A-HJ-NP-Za-km-z]{32}/g);
  if (!res) return false;
  return res[0] === _address;
};
var monitorUtxoByAddress$1 = function () {
  var _ref2 = _asyncToGenerator( _regeneratorRuntime.mark(function _callee2(_network, _address, _eventEmitter, _pollingTime, _broadcastEventName, _confirmationEventName) {
    var _confirmations,
        isBroadcasted,
        utxo,
        _args2 = arguments;
    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _confirmations = _args2.length > 6 && _args2[6] !== undefined ? _args2[6] : 1;
            isBroadcasted = false;
            utxo = null;
            _context2.next = 5;
            return polling( _asyncToGenerator( _regeneratorRuntime.mark(function _callee() {
              var utxos;
              return _regeneratorRuntime.wrap(function _callee$(_context) {
                while (1) {
                  switch (_context.prev = _context.next) {
                    case 0:
                      _context.next = 2;
                      return getUtxoByAddress$1(_network, _address);
                    case 2:
                      utxos = _context.sent;
                      if (!(utxos.length > 0)) {
                        _context.next = 19;
                        break;
                      }
                      if (!(utxos[0].confirmations > 0)) {
                        _context.next = 13;
                        break;
                      }
                      if (!isBroadcasted) {
                        _eventEmitter.emit(_broadcastEventName, utxos[0]);
                        isBroadcasted = true;
                      }
                      if (!(utxos[0].confirmations >= _confirmations)) {
                        _context.next = 10;
                        break;
                      }
                      _eventEmitter.emit(_confirmationEventName, utxos[0]);
                      utxo = utxos[0].tx_hash;
                      return _context.abrupt("return", true);
                    case 10:
                      return _context.abrupt("return", false);
                    case 13:
                      if (isBroadcasted) {
                        _context.next = 17;
                        break;
                      }
                      isBroadcasted = true;
                      _eventEmitter.emit(_broadcastEventName, utxos[0]);
                      return _context.abrupt("return", false);
                    case 17:
                      _context.next = 20;
                      break;
                    case 19:
                      return _context.abrupt("return", false);
                    case 20:
                    case "end":
                      return _context.stop();
                  }
                }
              }, _callee);
            })), _pollingTime);
          case 5:
            return _context2.abrupt("return", utxo.startsWith('0x') ? utxo : '0x' + utxo);
          case 6:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return function monitorUtxoByAddress(_x, _x2, _x3, _x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  };
}();
var waitForTransactionConfirmation$2 = function () {
  var _ref4 = _asyncToGenerator( _regeneratorRuntime.mark(function _callee4(_network, _transactionHash, _pollingTime) {
    var transactionToReturn;
    return _regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            transactionToReturn = null;
            _context4.next = 3;
            return polling( _asyncToGenerator( _regeneratorRuntime.mark(function _callee3() {
              var _yield$_makeDogeChain, transaction;
              return _regeneratorRuntime.wrap(function _callee3$(_context3) {
                while (1) {
                  switch (_context3.prev = _context3.next) {
                    case 0:
                      _context3.prev = 0;
                      _context3.next = 3;
                      return _makeDogeChainApiCall(_network, 'GET', "/transaction/".concat(_transactionHash, "/"));
                    case 3:
                      _yield$_makeDogeChain = _context3.sent;
                      transaction = _yield$_makeDogeChain.transaction;
                      transactionToReturn = transaction;
                      return _context3.abrupt("return", transaction.confirmations > 0);
                    case 9:
                      _context3.prev = 9;
                      _context3.t0 = _context3["catch"](0);
                      return _context3.abrupt("return", false);
                    case 12:
                    case "end":
                      return _context3.stop();
                  }
                }
              }, _callee3, null, [[0, 9]]);
            })), _pollingTime);
          case 3:
            return _context4.abrupt("return", transactionToReturn);
          case 4:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));
  return function waitForTransactionConfirmation(_x7, _x8, _x9) {
    return _ref4.apply(this, arguments);
  };
}();

var doge = /*#__PURE__*/Object.freeze({
  __proto__: null,
  broadcastTransaction: broadcastTransaction$1,
  isValidAddress: isValidAddress$2,
  getUtxoByAddress: getUtxoByAddress$1,
  monitorUtxoByAddress: monitorUtxoByAddress$1,
  waitForTransactionConfirmation: waitForTransactionConfirmation$2
});

var LTC_PTOKENS_NODE_TESTNET_API = 'https://ltc-testnet-node-1.ptokens.io/insight-lite-api';
var LTC_PTOKENS_NODE_MAINNET_API = 'https://ltc-node-1.ptokens.io/insight-lite-api';
var _getInsightLiteApi = function _getInsightLiteApi(_network) {
  return axios.create({
    baseURL: _network === Mainnet ? LTC_PTOKENS_NODE_MAINNET_API : LTC_PTOKENS_NODE_TESTNET_API,
    timeout: 50000,
    headers: {
      'Content-Type': 'application/json'
    }
  });
};
var _makeInsightLiteApiCall = function _makeInsightLiteApiCall(_network, _callType, _apiPath, _params) {
  return new Promise(function (resolve, reject) {
    _getInsightLiteApi(_network)[_callType.toLowerCase()](_apiPath, _params).then(function (_res) {
      return resolve(_res.data);
    }).catch(function (_err) {
      return reject(_err);
    });
  });
};
var broadcastTransaction$2 = function broadcastTransaction(_network, _tx) {
  return _makeInsightLiteApiCall(_network, 'POST', '/tx/send', {
    rawtx: _tx
  });
};
var getUtxoByAddress$2 = function getUtxoByAddress(_network, _address) {
  return _makeInsightLiteApiCall(_network, 'GET', "/addrs/".concat(_address, "/utxo"));
};
var getTransactionHexById$1 = function getTransactionHexById(_network, _txId) {
  return _makeInsightLiteApiCall(_network, 'GET', "/rawtx/".concat(_txId));
};
var isValidAddress$3 = function isValidAddress(_address) {
  var res = _address.match(/(ltc1|[LM3Q2mn])[a-zA-HJ-NP-Z0-9]{26,40}/g);
  if (!res) return false;
  return res[0] === _address;
};
var monitorUtxoByAddress$2 = function () {
  var _ref = _asyncToGenerator( _regeneratorRuntime.mark(function _callee2(_network, _address, _eventEmitter, _pollingTime, _broadcastEventName, _confirmationEventName) {
    var _confirmations,
        isBroadcasted,
        utxo,
        utxos,
        _args2 = arguments;
    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _confirmations = _args2.length > 6 && _args2[6] !== undefined ? _args2[6] : 1;
            isBroadcasted = false;
            utxo = null;
            utxos = [];
            _context2.next = 6;
            return polling( _asyncToGenerator( _regeneratorRuntime.mark(function _callee() {
              return _regeneratorRuntime.wrap(function _callee$(_context) {
                while (1) {
                  switch (_context.prev = _context.next) {
                    case 0:
                      _context.next = 2;
                      return _makeInsightLiteApiCall(_network, 'GET', "/addrs/".concat(_address, "/utxo"));
                    case 2:
                      utxos = _context.sent;
                      if (!(utxos.length > 0)) {
                        _context.next = 21;
                        break;
                      }
                      if (!(utxos[0].confirmations > 0)) {
                        _context.next = 14;
                        break;
                      }
                      if (!isBroadcasted) {
                        _eventEmitter.emit(_broadcastEventName, utxos[0]);
                        _eventEmitter.emit('onLtcTxBroadcasted', utxos[0]);
                        isBroadcasted = true;
                      }
                      if (!(utxos[0].confirmations >= _confirmations)) {
                        _context.next = 11;
                        break;
                      }
                      _eventEmitter.emit(_confirmationEventName, utxos[0]);
                      _eventEmitter.emit('onLtcTxConfirmed', utxos[0]);
                      utxo = utxos[0].txid;
                      return _context.abrupt("return", true);
                    case 11:
                      return _context.abrupt("return", false);
                    case 14:
                      if (isBroadcasted) {
                        _context.next = 19;
                        break;
                      }
                      isBroadcasted = true;
                      _eventEmitter.emit(_broadcastEventName, utxos[0]);
                      _eventEmitter.emit('onLtcTxBroadcasted', utxos[0]);
                      return _context.abrupt("return", false);
                    case 19:
                      _context.next = 22;
                      break;
                    case 21:
                      return _context.abrupt("return", false);
                    case 22:
                    case "end":
                      return _context.stop();
                  }
                }
              }, _callee);
            })), _pollingTime);
          case 6:
            return _context2.abrupt("return", utxo);
          case 7:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return function monitorUtxoByAddress(_x, _x2, _x3, _x4, _x5, _x6) {
    return _ref.apply(this, arguments);
  };
}();
var waitForTransactionConfirmation$3 = function () {
  var _ref3 = _asyncToGenerator( _regeneratorRuntime.mark(function _callee4(_network, _tx, _pollingTime) {
    var transaction;
    return _regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            transaction = null;
            _context4.next = 3;
            return polling( _asyncToGenerator( _regeneratorRuntime.mark(function _callee3() {
              return _regeneratorRuntime.wrap(function _callee3$(_context3) {
                while (1) {
                  switch (_context3.prev = _context3.next) {
                    case 0:
                      _context3.prev = 0;
                      _context3.next = 3;
                      return _makeInsightLiteApiCall(_network, 'GET', "/tx/".concat(_tx, "/"));
                    case 3:
                      transaction = _context3.sent;
                      return _context3.abrupt("return", transaction.confirmations > 0);
                    case 7:
                      _context3.prev = 7;
                      _context3.t0 = _context3["catch"](0);
                      return _context3.abrupt("return", false);
                    case 10:
                    case "end":
                      return _context3.stop();
                  }
                }
              }, _callee3, null, [[0, 7]]);
            })), _pollingTime);
          case 3:
            return _context4.abrupt("return", transaction);
          case 4:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));
  return function waitForTransactionConfirmation(_x7, _x8, _x9) {
    return _ref3.apply(this, arguments);
  };
}();

var ltc = /*#__PURE__*/Object.freeze({
  __proto__: null,
  broadcastTransaction: broadcastTransaction$2,
  isValidAddress: isValidAddress$3,
  getUtxoByAddress: getUtxoByAddress$2,
  getTransactionHexById: getTransactionHexById$1,
  monitorUtxoByAddress: monitorUtxoByAddress$2,
  waitForTransactionConfirmation: waitForTransactionConfirmation$3
});

var UINT32_MAX = Math.pow(2, 32);
var decodeUint64le = function decodeUint64le(_buffer) {
  var rem = _buffer.readUInt32LE(0);
  var top = _buffer.readUInt32LE(4);
  return top * UINT32_MAX + rem;
};
var encodeUint64le = function encodeUint64le(_number) {
  var buffer = Buffer.alloc(8);
  var top = Math.floor(_number / UINT32_MAX);
  var rem = _number - top * UINT32_MAX;
  buffer.writeUInt32LE(rem, 0);
  buffer.writeUInt32LE(top, 4);
  return buffer;
};

var converters = /*#__PURE__*/Object.freeze({
  __proto__: null,
  decodeUint64le: decodeUint64le,
  encodeUint64le: encodeUint64le
});

var HEX_PREFIX = '0x';
var zeroEther = '0x00';
var zeroAddress = '0x0000000000000000000000000000000000000000';
var addHexPrefix = function addHexPrefix(_string) {
  return isHexPrefixed(_string) ? _string : HEX_PREFIX + _string;
};
var removeHexPrefix = function removeHexPrefix(_string) {
  return isHexPrefixed(_string) ? _string.substr(2) : _string;
};
var onChainFormat = function onChainFormat(_amount, _decimals) {
  return _amount.multipliedBy(new BigNumber(Math.pow(10, _decimals)));
};
var offChainFormat = function offChainFormat(_amount, _decimals) {
  return _amount.dividedBy(new BigNumber(Math.pow(10, _decimals)));
};
var getAccount = function getAccount(_web3) {
  return new Promise(function (_resolve, _reject) {
    _web3.eth.defaultAccount ? _resolve(_web3.eth.defaultAccount) : _web3.eth.getAccounts().then(function (accounts) {
      return _resolve(accounts[0]);
    }).catch(function (err) {
      return _reject(err);
    });
  });
};
var getContract = function getContract(_web3, _abi, _contractAddress, _account) {
  var contract = new _web3.eth.Contract(_abi, _contractAddress, {
    defaultAccount: _account
  });
  return contract;
};
var getGasLimit = function getGasLimit(_web3) {
  return new Promise(function (_resolve, _reject) {
    _web3.eth.getBlock('latest').then(function (_block) {
      return _resolve(_block.gasLimit);
    }).catch(function (_err) {
      return _reject(_err);
    });
  });
};
var isHexPrefixed = function isHexPrefixed(_string) {
  return _string.slice(0, 2) === HEX_PREFIX;
};
var makeContractCall = function () {
  var _ref = _asyncToGenerator( _regeneratorRuntime.mark(function _callee(_web3, _method, _options) {
    var _params,
        _contract$methods,
        abi,
        contractAddress,
        account,
        contract,
        _args = arguments;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _params = _args.length > 3 && _args[3] !== undefined ? _args[3] : [];
            _context.prev = 1;
            abi = _options.abi, contractAddress = _options.contractAddress;
            _context.next = 5;
            return getAccount(_web3);
          case 5:
            account = _context.sent;
            contract = getContract(_web3, abi, contractAddress, account);
            return _context.abrupt("return", (_contract$methods = contract.methods)[_method].apply(_contract$methods, _toConsumableArray(_params)).call());
          case 10:
            _context.prev = 10;
            _context.t0 = _context["catch"](1);
            throw new Error(_context.t0.message);
          case 13:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[1, 10]]);
  }));
  return function makeContractCall(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();
var makeContractSend = function makeContractSend(_web3, _method, _options) {
  var _params = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];
  var promiEvent = Web3PromiEvent();
  var start = function () {
    var _ref2 = _asyncToGenerator( _regeneratorRuntime.mark(function _callee2() {
      var _contract$methods2, abi, contractAddress, value, gasPrice, gas, account, contract;
      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.prev = 0;
              abi = _options.abi, contractAddress = _options.contractAddress, value = _options.value, gasPrice = _options.gasPrice, gas = _options.gas;
              _context2.next = 4;
              return getAccount(_web3);
            case 4:
              account = _context2.sent;
              contract = getContract(_web3, abi, contractAddress, account);
              (_contract$methods2 = contract.methods)[_method].apply(_contract$methods2, _toConsumableArray(_params)).send({
                from: account,
                value: value,
                gasPrice: gasPrice,
                gas: gas
              }).once('transactionHash', function (_hash) {
                return promiEvent.eventEmitter.emit('transactionHash', _hash);
              }).once('receipt', function (_receipt) {
                return promiEvent.eventEmitter.emit('receipt', _receipt);
              }).once('error', function (_error) {
                return promiEvent.eventEmitter.emit('error', _error);
              }).then(function () {
                return promiEvent.resolve();
              });
              _context2.next = 12;
              break;
            case 9:
              _context2.prev = 9;
              _context2.t0 = _context2["catch"](0);
              promiEvent.reject(_context2.t0);
            case 12:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, null, [[0, 9]]);
    }));
    return function start() {
      return _ref2.apply(this, arguments);
    };
  }();
  start();
  return promiEvent.eventEmitter;
};
var sendSignedMethodTx = function sendSignedMethodTx(_web3, _method, _options, _params) {
  var promiEvent = Web3PromiEvent();
  var start = function () {
    var _ref3 = _asyncToGenerator( _regeneratorRuntime.mark(function _callee3() {
      var _contract$methods3, abi, contractAddress, value, gas, gasPrice, privateKey, contract, nonce, _yield$_web3$eth$acco, rawTransaction;
      return _regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.prev = 0;
              abi = _options.abi, contractAddress = _options.contractAddress, value = _options.value, gas = _options.gas, gasPrice = _options.gasPrice, privateKey = _options.privateKey;
              contract = getContract(_web3, abi, _web3.eth.defaultAccount);
              _context3.next = 5;
              return _web3.eth.getTransactionCount(_web3.eth.defaultAccount, 'pending');
            case 5:
              nonce = _context3.sent;
              _context3.t0 = _web3.eth.accounts;
              _context3.t1 = nonce;
              _context3.t2 = gasPrice;
              if (_context3.t2) {
                _context3.next = 13;
                break;
              }
              _context3.next = 12;
              return _web3.eth.getGasPrice();
            case 12:
              _context3.t2 = _context3.sent;
            case 13:
              _context3.t3 = _context3.t2;
              _context3.t4 = gas;
              if (_context3.t4) {
                _context3.next = 19;
                break;
              }
              _context3.next = 18;
              return getGasLimit(_web3);
            case 18:
              _context3.t4 = _context3.sent;
            case 19:
              _context3.t5 = _context3.t4;
              _context3.t6 = contractAddress;
              _context3.t7 = value;
              _context3.t8 = (_contract$methods3 = contract.methods)[_method].apply(_contract$methods3, _toConsumableArray(_params)).encodeABI();
              _context3.t9 = {
                nonce: _context3.t1,
                gasPrice: _context3.t3,
                gasLimit: _context3.t5,
                to: _context3.t6,
                value: _context3.t7,
                data: _context3.t8
              };
              _context3.t10 = privateKey;
              _context3.next = 27;
              return _context3.t0.signTransaction.call(_context3.t0, _context3.t9, _context3.t10);
            case 27:
              _yield$_web3$eth$acco = _context3.sent;
              rawTransaction = _yield$_web3$eth$acco.rawTransaction;
              _web3.eth.sendSignedTransaction(rawTransaction).once('transactionHash', function (_hash) {
                return promiEvent.eventEmitter.emit('transactionHash', _hash);
              }).once('receipt', function (_receipt) {
                return promiEvent.eventEmitter.emit('receipt', _receipt);
              }).once('error', function (_error) {
                return promiEvent.eventEmitter.emit('error', _error);
              }).then(function () {
                return promiEvent.resolve();
              });
              _context3.next = 35;
              break;
            case 32:
              _context3.prev = 32;
              _context3.t11 = _context3["catch"](0);
              promiEvent.reject(_context3.t11);
            case 35:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3, null, [[0, 32]]);
    }));
    return function start() {
      return _ref3.apply(this, arguments);
    };
  }();
  start();
  return promiEvent.eventEmitter;
};
var waitForTransactionConfirmation$4 = function () {
  var _ref4 = _asyncToGenerator( _regeneratorRuntime.mark(function _callee5(_web3, _tx) {
    var _pollingTime,
        receipt,
        _args5 = arguments;
    return _regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _pollingTime = _args5.length > 2 && _args5[2] !== undefined ? _args5[2] : 5000;
            receipt = null;
            _context5.next = 4;
            return polling( _asyncToGenerator( _regeneratorRuntime.mark(function _callee4() {
              return _regeneratorRuntime.wrap(function _callee4$(_context4) {
                while (1) {
                  switch (_context4.prev = _context4.next) {
                    case 0:
                      _context4.prev = 0;
                      _context4.next = 3;
                      return _web3.eth.getTransactionReceipt(_tx);
                    case 3:
                      receipt = _context4.sent;
                      if (receipt) {
                        _context4.next = 8;
                        break;
                      }
                      return _context4.abrupt("return", false);
                    case 8:
                      if (!receipt.status) {
                        _context4.next = 12;
                        break;
                      }
                      return _context4.abrupt("return", true);
                    case 12:
                      return _context4.abrupt("return", false);
                    case 13:
                      _context4.next = 18;
                      break;
                    case 15:
                      _context4.prev = 15;
                      _context4.t0 = _context4["catch"](0);
                      return _context4.abrupt("return", false);
                    case 18:
                    case "end":
                      return _context4.stop();
                  }
                }
              }, _callee4, null, [[0, 15]]);
            })), _pollingTime);
          case 4:
            return _context5.abrupt("return", receipt);
          case 5:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5);
  }));
  return function waitForTransactionConfirmation(_x4, _x5) {
    return _ref4.apply(this, arguments);
  };
}();

var eth = /*#__PURE__*/Object.freeze({
  __proto__: null,
  addHexPrefix: addHexPrefix,
  removeHexPrefix: removeHexPrefix,
  onChainFormat: onChainFormat,
  offChainFormat: offChainFormat,
  getAccount: getAccount,
  getContract: getContract,
  getGasLimit: getGasLimit,
  isHexPrefixed: isHexPrefixed,
  makeContractCall: makeContractCall,
  makeContractSend: makeContractSend,
  sendSignedMethodTx: sendSignedMethodTx,
  waitForTransactionConfirmation: waitForTransactionConfirmation$4,
  zeroEther: zeroEther,
  zeroAddress: zeroAddress
});

var EOS_TRANSACTION_EXECUTED = 'executed';
var getApi = function getApi(_privateKey, _rpc) {
  var _signatureProvider = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
  if (_rpc && !_privateKey && !_signatureProvider) {
    var _rpc2 = typeof _rpc === 'string' ? new eosjs.JsonRpc(_rpc, {
      fetch: fetch
    }) : _rpc;
    return new eosjs.Api({
      rpc: _rpc2,
      textDecoder: new encoding.TextDecoder(),
      textEncoder: new encoding.TextEncoder()
    });
  }
  var signatureProvider = _signatureProvider || new eosjsJssig.JsSignatureProvider([_privateKey]);
  var rpc = typeof _rpc === 'string' ? new eosjs.JsonRpc(_rpc, {
    fetch: fetch
  }) : _rpc;
  return new eosjs.Api({
    rpc: rpc,
    signatureProvider: signatureProvider,
    textDecoder: new encoding.TextDecoder(),
    textEncoder: new encoding.TextEncoder()
  });
};
var getAmountInEosFormat = function getAmountInEosFormat(_amount) {
  var _decimals = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 4;
  var symbol = arguments.length > 2 ? arguments[2] : undefined;
  return "".concat(parseFloat(_amount).toFixed(_decimals), " ").concat(symbol);
};
var isValidAccountName = function isValidAccountName(_accountName) {
  return new RegExp('(^[a-z1-5.]{0,11}[a-z1-5]$)|(^[a-z1-5.]{12}[a-j1-5]$)').test(_accountName);
};
var waitForTransactionConfirmation$5 = function () {
  var _ref = _asyncToGenerator( _regeneratorRuntime.mark(function _callee2(_api, _tx) {
    var _pollingTime,
        receipt,
        _args2 = arguments;
    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _pollingTime = _args2.length > 2 && _args2[2] !== undefined ? _args2[2] : 2000;
            receipt = null;
            _context2.next = 4;
            return polling( _asyncToGenerator( _regeneratorRuntime.mark(function _callee() {
              return _regeneratorRuntime.wrap(function _callee$(_context) {
                while (1) {
                  switch (_context.prev = _context.next) {
                    case 0:
                      _context.prev = 0;
                      _context.next = 3;
                      return _api.rpc.history_get_transaction(_tx);
                    case 3:
                      receipt = _context.sent;
                      if (!(receipt && receipt.trx.receipt.status === EOS_TRANSACTION_EXECUTED)) {
                        _context.next = 8;
                        break;
                      }
                      return _context.abrupt("return", true);
                    case 8:
                      return _context.abrupt("return", false);
                    case 9:
                      _context.next = 14;
                      break;
                    case 11:
                      _context.prev = 11;
                      _context.t0 = _context["catch"](0);
                      return _context.abrupt("return", false);
                    case 14:
                    case "end":
                      return _context.stop();
                  }
                }
              }, _callee, null, [[0, 11]]);
            })), _pollingTime);
          case 4:
            return _context2.abrupt("return", receipt);
          case 5:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return function waitForTransactionConfirmation(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

var eos = /*#__PURE__*/Object.freeze({
  __proto__: null,
  getApi: getApi,
  getAmountInEosFormat: getAmountInEosFormat,
  isValidAccountName: isValidAccountName,
  waitForTransactionConfirmation: waitForTransactionConfirmation$5
});

var RVN_PTOKENS_NODE_MAINNET_API = 'https://corsproxy.ptokens.io/v1/?apiurl=https://api.ravencoin.org/api';
var _getInsightLiteApi$1 = function _getInsightLiteApi(_network) {
  if (_network === Testnet) throw new Error('Ravecoin Testnet is not supported');
  return axios.create({
    baseURL: RVN_PTOKENS_NODE_MAINNET_API,
    timeout: 50000,
    headers: {
      'Content-Type': 'application/json'
    }
  });
};
var _makeInsightLiteApiCall$1 = function _makeInsightLiteApiCall(_network, _callType, _apiPath, _params) {
  return new Promise(function (resolve, reject) {
    _getInsightLiteApi$1(_network)[_callType.toLowerCase()](_apiPath, _params).then(function (_res) {
      return resolve(_res.data);
    }).catch(function (_err) {
      return reject(_err);
    });
  });
};
var broadcastTransaction$3 = function broadcastTransaction(_network, _tx) {
  return _makeInsightLiteApiCall$1(_network, 'POST', '/tx/send', {
    rawtx: _tx
  });
};
var getUtxoByAddress$3 = function getUtxoByAddress(_network, _address) {
  return _makeInsightLiteApiCall$1(_network, 'GET', "/addrs/".concat(_address, "/utxo"));
};
var getTransactionHexById$2 = function getTransactionHexById(_network, _txId) {
  return _makeInsightLiteApiCall$1(_network, 'GET', "/rawtx/".concat(_txId));
};
var isValidAddress$4 = function isValidAddress(_address) {
  var res = _address.match(/(r|R)[a-zA-HJ-NP-Z0-9]{26,40}/g);
  if (!res) return false;
  return res[0] === _address;
};
var monitorUtxoByAddress$3 = function () {
  var _ref = _asyncToGenerator( _regeneratorRuntime.mark(function _callee2(_network, _address, _eventEmitter, _pollingTime, _broadcastEventName, _confirmationEventName) {
    var _confirmations,
        isBroadcasted,
        utxo,
        utxos,
        _args2 = arguments;
    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _confirmations = _args2.length > 6 && _args2[6] !== undefined ? _args2[6] : 1;
            isBroadcasted = false;
            utxo = null;
            utxos = [];
            _context2.next = 6;
            return polling( _asyncToGenerator( _regeneratorRuntime.mark(function _callee() {
              return _regeneratorRuntime.wrap(function _callee$(_context) {
                while (1) {
                  switch (_context.prev = _context.next) {
                    case 0:
                      _context.next = 2;
                      return _makeInsightLiteApiCall$1(_network, 'GET', "/addrs/".concat(_address, "/utxo"));
                    case 2:
                      utxos = _context.sent;
                      if (!(utxos.length > 0)) {
                        _context.next = 19;
                        break;
                      }
                      if (!(utxos[0].confirmations > 0)) {
                        _context.next = 13;
                        break;
                      }
                      if (!isBroadcasted) {
                        _eventEmitter.emit(_broadcastEventName, utxos[0]);
                        isBroadcasted = true;
                      }
                      if (!(utxos[0].confirmations >= _confirmations)) {
                        _context.next = 10;
                        break;
                      }
                      _eventEmitter.emit(_confirmationEventName, utxos[0]);
                      utxo = utxos[0].txid;
                      return _context.abrupt("return", true);
                    case 10:
                      return _context.abrupt("return", false);
                    case 13:
                      if (isBroadcasted) {
                        _context.next = 17;
                        break;
                      }
                      isBroadcasted = true;
                      _eventEmitter.emit(_broadcastEventName, utxos[0]);
                      return _context.abrupt("return", false);
                    case 17:
                      _context.next = 20;
                      break;
                    case 19:
                      return _context.abrupt("return", false);
                    case 20:
                    case "end":
                      return _context.stop();
                  }
                }
              }, _callee);
            })), _pollingTime);
          case 6:
            return _context2.abrupt("return", utxo);
          case 7:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return function monitorUtxoByAddress(_x, _x2, _x3, _x4, _x5, _x6) {
    return _ref.apply(this, arguments);
  };
}();
var waitForTransactionConfirmation$6 = function () {
  var _ref3 = _asyncToGenerator( _regeneratorRuntime.mark(function _callee4(_network, _tx) {
    var _pollingTime,
        transaction,
        _args4 = arguments;
    return _regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _pollingTime = _args4.length > 2 && _args4[2] !== undefined ? _args4[2] : 3000;
            transaction = null;
            _context4.next = 4;
            return polling( _asyncToGenerator( _regeneratorRuntime.mark(function _callee3() {
              return _regeneratorRuntime.wrap(function _callee3$(_context3) {
                while (1) {
                  switch (_context3.prev = _context3.next) {
                    case 0:
                      _context3.prev = 0;
                      _context3.next = 3;
                      return _makeInsightLiteApiCall$1(_network, 'GET', "/tx/".concat(_tx, "/"));
                    case 3:
                      transaction = _context3.sent;
                      return _context3.abrupt("return", transaction.confirmations > 0);
                    case 7:
                      _context3.prev = 7;
                      _context3.t0 = _context3["catch"](0);
                      return _context3.abrupt("return", false);
                    case 10:
                    case "end":
                      return _context3.stop();
                  }
                }
              }, _callee3, null, [[0, 7]]);
            })), _pollingTime);
          case 4:
            return _context4.abrupt("return", transaction);
          case 5:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));
  return function waitForTransactionConfirmation(_x7, _x8) {
    return _ref3.apply(this, arguments);
  };
}();

var rvn = /*#__PURE__*/Object.freeze({
  __proto__: null,
  broadcastTransaction: broadcastTransaction$3,
  isValidAddress: isValidAddress$4,
  getUtxoByAddress: getUtxoByAddress$3,
  getTransactionHexById: getTransactionHexById$2,
  monitorUtxoByAddress: monitorUtxoByAddress$3,
  waitForTransactionConfirmation: waitForTransactionConfirmation$6
});

var _pTokenNativeBlockcha;
var blockchainTypes = {
  ethereum: Ethereum,
  eth: Ethereum,
  eosio: Eosio,
  eos: Eosio,
  bitcoin: Bitcoin,
  btc: Bitcoin,
  ltc: Litecoin,
  litecoin: Litecoin,
  telos: Telos,
  dogecoin: Dogecoin,
  doge: Dogecoin,
  'binance-smart-chain': BinanceSmartChain,
  bsc: BinanceSmartChain,
  matic: Polygon,
  polygon: Polygon,
  xdai: Xdai,
  ravencoin: Ravencoin,
  rvn: Ravencoin,
  lbc: Lbry,
  lbry: Lbry,
  ultra: Ultra,
  uos: Ultra,
  arbitrum: Arbitrum,
  luxochain: Luxochain,
  algorand: Algorand,
  fantom: Fantom,
  ore: Ore,
  libre: Libre
};
var blockchainShortTypes = {
  ethereum: 'eth',
  eosio: 'eos',
  bitcoin: 'btc',
  litecoin: 'ltc',
  telos: 'telos',
  dogecoin: 'doge',
  'binance-smart-chain': 'bsc',
  matic: 'polygon',
  polygon: 'polygon',
  xdai: 'xdai',
  ravencoin: 'rvn',
  lbc: 'lbc',
  lbry: 'lbc',
  ultra: 'ultra',
  arbitrum: 'arbitrum',
  luxochain: 'luxochain',
  algorand: 'algo',
  fantom: 'fantom',
  ftm: 'fantom',
  ore: 'ore',
  libre: 'libre'
};
var pTokenNativeBlockchain = (_pTokenNativeBlockcha = {
  pbtc: Bitcoin,
  pltc: Litecoin,
  pweth: Ethereum,
  peth: Ethereum,
  plink: Ethereum,
  pyfi: Ethereum,
  pmkr: Ethereum,
  pnt: Ethereum,
  pteria: Ethereum,
  puni: Ethereum,
  pband: Ethereum,
  pbal: Ethereum,
  pcomp: Ethereum,
  psnx: Ethereum,
  pomg: Ethereum,
  pdai: Ethereum,
  pant: Ethereum,
  plrc: Ethereum,
  puos: Ethereum,
  pbat: Ethereum,
  prep: Ethereum,
  pzrx: Ethereum,
  ppnk: Ethereum,
  pdoge: Dogecoin,
  peos: Eosio,
  iq: Eosio,
  tlos: Telos,
  popium: Ethereum,
  pbcp: Ethereum,
  pdefiplusplus: Ethereum,
  cgg: Ethereum,
  pusdc: Ethereum,
  pusdt: Ethereum,
  prvn: Ravencoin,
  popen: Ethereum,
  ocp: BinanceSmartChain,
  anrx: Ethereum,
  tff: BinanceSmartChain,
  psafemoon: BinanceSmartChain,
  efx: Eosio,
  pseeds: Telos,
  plbc: Lbry,
  usdo: BinanceSmartChain,
  gala: Ethereum,
  pzmt: Ethereum,
  bist: Ethereum,
  pvai: BinanceSmartChain,
  wsb: BinanceSmartChain,
  luxo: Luxochain,
  ptet: BinanceSmartChain,
  pkeys: Ethereum,
  oath: Fantom,
  pftm: Fantom,
  pwftm: Fantom,
  nuco: Ethereum
}, _defineProperty(_pTokenNativeBlockcha, "pkeys", Ethereum), _defineProperty(_pTokenNativeBlockcha, "ore", Ore), _defineProperty(_pTokenNativeBlockcha, "libre", Libre), _pTokenNativeBlockcha);
var networkLabels = {
  ethereum: {
    testnet: EthereumRopsten,
    testnet_ropsten: EthereumRopsten,
    mainnet: EthereumMainnet
  },
  eosio: {
    testnet: EosioJungle3,
    testnet_jungle3: EosioJungle3,
    mainnet: EosioMainnet
  },
  bitcoin: {
    bitcoin: Bitcoin,
    mainnet: BitcoinMainnet,
    testnet: BitcoinTestnet
  },
  litecoin: {
    testnet: LitecoinTestnet,
    mainnet: LitecoinMainnet,
    litecoin: LitecoinMainnet
  },
  telos: {
    mainnet: TelosMainnet
  },
  dogecoin: {
    mainnet: DogecoinMainnet,
    dogecoin: DogecoinMainnet
  },
  'binance-smart-chain': {
    mainnet: BinanceSmartChainMainnet
  },
  polygon: {
    mainnet: PolygonMainnet
  },
  matic: {
    mainnet: PolygonMainnet
  },
  xdai: {
    mainnet: XdaiMainnet
  },
  ravencoin: {
    mainnet: RavencoinMainnet,
    ravencoin: RavencoinMainnet
  },
  lbry: {
    mainnet: LbryMainnet
  },
  ultra: {
    mainnet: UltraMainnet,
    testnet: UltraTestnet
  },
  arbitrum: {
    mainnet: ArbitrumMainnet
  },
  luxochain: {
    mainnet: LuxochainMainnet
  },
  algorand: {
    mainnet: AlgorandMainnet
  },
  fantom: {
    mainnet: FantomMainnet
  },
  ore: {
    mainnet: OreMainnet
  },
  libre: {
    mainnet: LibreMainnet
  }
};
var networkLabelType = {
  testnet_ropsten: Testnet,
  testnet_jungle3: Testnet,
  testnet: Testnet,
  mainnet: Mainnet
};
var pTokensAvailables = [pBTC, pLTC, pWETH, pETH, pYFI, pMKR, PNT, pLINK, PTERIA, pUNI, pBAND, pBAL, pCOMP, pSNX, pOMG, pDAI, pANT, pLRC, pUOS, pBAT, pREP, pZRX, pPNK, pDOGE, pEOS, IQ, TLOS, pOPIUM, pBCP, pDEFIPlusPlus, CGG, pUSDT, pUSDC, pRVN, pOPEN, OCP, ANRX, TFF, pSAFEMOON, EFX, pSEEDS, pLBC, USDO, GALA, ZMT, BIST, pVAI, WSB, LUXO, pTET, pKEYS, OATH, pFTM, pWFTM, NUCO, ORE];

var getNetworkType = function getNetworkType(_network) {
  return networkLabelType[_network.toLowerCase()];
};
var getBlockchainType = function getBlockchainType(_blockchain) {
  return blockchainTypes[_blockchain.toLowerCase()];
};
var getBlockchainShortType = function getBlockchainShortType(_blockchain) {
  return blockchainShortTypes[_blockchain.toLowerCase()];
};
var getNativeBlockchainFromPtokenName = function getNativeBlockchainFromPtokenName(_name) {
  return pTokenNativeBlockchain[_name.toLowerCase()];
};
var isValidPTokenName = function isValidPTokenName(_name) {
  return Boolean(pTokensAvailables.includes(_name.toLowerCase()));
};
var parseParams = function parseParams(_params, _nativeBlockchain) {
  var parsedHostBlockchain;
  var parsedHostNetwork;
  var parsedNativeBlockchain;
  var parsedNativeNetwork;
  var blockchain = _params.blockchain,
      network = _params.network,
      hostBlockchain = _params.hostBlockchain,
      hostNetwork = _params.hostNetwork,
      nativeBlockchain = _params.nativeBlockchain,
      nativeNetwork = _params.nativeNetwork;
  if (Boolean(hostBlockchain) === Boolean(blockchain) || Boolean(hostNetwork) === Boolean(network)) throw new Error('Bad initialization');
  if (hostBlockchain) parsedHostBlockchain = blockchainTypes[hostBlockchain.toLowerCase()];else if (blockchain) parsedHostBlockchain = blockchainTypes[blockchain.toLowerCase()];
  if (!parsedHostBlockchain) throw new Error('Invalid hostBlockchain value');
  if (hostNetwork) parsedHostNetwork = networkLabels[parsedHostBlockchain][hostNetwork.toLowerCase()];else if (network) parsedHostNetwork = networkLabels[parsedHostBlockchain][network.toLowerCase()];
  if (!parsedHostNetwork) throw new Error('Invalid hostNetwork value');
  parsedNativeNetwork = nativeNetwork ? networkLabels[nativeNetwork.toLowerCase()] : null;
  if (!parsedNativeNetwork && parsedHostNetwork.includes(Testnet)) parsedNativeNetwork = Testnet;else parsedNativeNetwork = Mainnet;
  parsedNativeBlockchain = nativeBlockchain ? blockchainTypes[nativeBlockchain.toLowerCase()] : _nativeBlockchain;
  return {
    hostBlockchain: parsedHostBlockchain,
    hostNetwork: parsedHostNetwork,
    nativeBlockchain: parsedNativeBlockchain,
    nativeNetwork: parsedNativeNetwork
  };
};

var helpers = /*#__PURE__*/Object.freeze({
  __proto__: null,
  getNetworkType: getNetworkType,
  getBlockchainType: getBlockchainType,
  getBlockchainShortType: getBlockchainShortType,
  getNativeBlockchainFromPtokenName: getNativeBlockchainFromPtokenName,
  isValidPTokenName: isValidPTokenName,
  parseParams: parseParams
});

var _Ethereum, _Ethereum$BinanceSmar;
var tokens = (_Ethereum$BinanceSmar = {}, _defineProperty(_Ethereum$BinanceSmar, Ethereum, (_Ethereum = {}, _defineProperty(_Ethereum, EthereumMainnet, {
  ETH: '0x0000000000000000000000000000000000000000',
  WETH: '0xc02aaa39b223fe8d0a0e5c4f27ead9083c756cc2',
  PNT: '0x89ab32156e46f46d02ade3fecbe5fc4243b9aaed',
  MKR: '0x9f8f72aa9304c8b593d555f12ef6589cc3a579a2',
  LINK: '0x514910771af9ca656af840dff83e8264ecf986ca',
  YFI: '0x0bc529c00C6401aEF6D220BE8C6Ea1667F6Ad93e',
  PTERIA: '0x02eca910cb3a7d43ebc7e8028652ed5c6b70259b',
  UNI: '0x1f9840a85d5af5bf1d1762f925bdaddc4201f984',
  BAND: '0xba11d00c5f74255f56a5e366f4f77f5a186d7f55',
  BAL: '0xba100000625a3754423978a60c9317c58a424e3d',
  COMP: '0xc00e94cb662c3520282e6f5717214004a7f26888',
  SNX: '0xc011a73ee8576fb46f5e1c5751ca3b9fe0af2a6f',
  OMG: '0xd26114cd6ee289accf82350c8d8487fedb8a0c07',
  DAI: '0x6b175474e89094c44da98b954eedeac495271d0f',
  ANT: '0xa117000000f279d81a1d3cc75430faa017fa5a2e',
  LRC: '0xbbbbca6a901c926f240b89eacb641d8aec7aeafd',
  UOS: '0xd13c7342e1ef687c5ad21b27c2b65d772cab5c8c',
  BAT: '0x0d8775f648430679a709e98d2b0cb6250d2887ef',
  REP: '0x221657776846890989a759ba2973e427dff5c9bb',
  ZRX: '0xe41d2489571d322189246dafa5ebde1f4699f498',
  PNK: '0x93ed3fbe21207ec2e8f2d3c3de6e058cb73bc04d',
  OPIUM: '0x888888888889c00c67689029d7856aac1065ec11',
  BCP: '0xe4f726adc8e89c6a6017f01eada77865db22da14',
  'DEFI++': '0x8d1ce361eb68e9e05573443c407d4a3bed23b033',
  CGG: '0x1fe24f25b1cf609b9c4e7e12d802e3640dfa5e43',
  USDT: '0xdac17f958d2ee523a2206206994597c13d831ec7',
  USDC: '0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48',
  OPEN: '0x69e8b9528cabda89fe846c67675b5d73d463a916',
  ANRX: '0xcae72a7a0fd9046cf6b165ca54c9e3a3872109e0',
  GALA: '0x15d4c048f83bd7e37d49ea4c83a07267ec4203da',
  ZMT: '0xaa602dE53347579f86b996D2Add74bb6F79462b2',
  BIST: '0x6e8908cfa881c9f6f2c64d3436e7b80b1bf0093f',
  LUXO: '0xd39a2cecba2657e125ba6a5c98ad2f6b6d7e83fd',
  KEYS: '0xe0a189C975e4928222978A74517442239a0b86ff',
  NCDT: '0xE0C8b298db4cfFE05d1bEA0bb1BA414522B33C1B'
}), _defineProperty(_Ethereum, EthereumRopsten, {
  UOS: '0x92829bc1a5b405fe14a0b7c38a5bad9a91b1dc02'
}), _Ethereum)), _defineProperty(_Ethereum$BinanceSmar, BinanceSmartChain, _defineProperty({}, BinanceSmartChainMainnet, {
  OCP: '0x3c70260eee0a2bfc4b375feb810325801f289fbd',
  TFF: '0x2d69c55baecefc6ec815239da0a985747b50db6e',
  SAFEMOON: '0x8076c74c5e3f5852037f31ff0093eeb8c8add8d3',
  USDO: '0x5801d0e1c7d977d78e4890880b8e579eb4943276',
  VAI: '0x4bd17003473389a42daf6a0a729f6fdb328bbbd7',
  WSB: '0x8244609023097aef71c702ccbaefc0bde5b48694',
  pTET: '0x87b92df541869fcabb7e5d4e3bfc3025eb3e7577'
})), _defineProperty(_Ethereum$BinanceSmar, Eosio, _defineProperty({}, EosioMainnet, {
  EOS: 'eosio.token',
  IQ: 'everipediaiq',
  EFX: 'effecttokens'
})), _defineProperty(_Ethereum$BinanceSmar, Telos, _defineProperty({}, TelosMainnet, {
  TLOS: 'eosio.token',
  SEEDS: 'token.seeds'
})), _defineProperty(_Ethereum$BinanceSmar, Fantom, _defineProperty({}, FantomMainnet, {
  OATH: '0x21ada0d2ac28c3a5fa3cd2ee30882da8812279b6'
})), _defineProperty(_Ethereum$BinanceSmar, Ore, _defineProperty({}, OreMainnet, {
  ORE: 'eosio.token'
})), _Ethereum$BinanceSmar);

var _Ethereum$1, _Bitcoin, _chainIds;
var blockchains = {
  Bitcoin: Bitcoin,
  Eosio: Eosio,
  Ethereum: Ethereum,
  Litecoin: Litecoin,
  Telos: Telos,
  Dogecoin: Dogecoin,
  BinanceSmartChain: BinanceSmartChain,
  Polygon: Polygon,
  Xdai: Xdai,
  Ravencoin: Ravencoin,
  Lbry: Lbry,
  Ultra: Ultra,
  Arbitrum: Arbitrum,
  Luxochain: Luxochain,
  Algorand: Algorand,
  Fantom: Fantom,
  Ore: Ore,
  Libre: Libre
};
var networks = {
  Mainnet: Mainnet,
  Testnet: Testnet,
  BitcoinMainnet: BitcoinMainnet,
  BitcoinTestnet: BitcoinTestnet,
  EthereumMainnet: EthereumMainnet,
  EthereumRopsten: EthereumRopsten,
  EosioMainnet: EosioMainnet,
  EosioJungle3: EosioJungle3,
  LitecoinMainnet: LitecoinMainnet,
  LitecoinTestnet: LitecoinTestnet,
  TelosMainnet: TelosMainnet,
  DogecoinMainnet: DogecoinMainnet,
  BinanceSmartChainMainnet: BinanceSmartChainMainnet,
  PolygonMainnet: PolygonMainnet,
  XdaiMainnet: XdaiMainnet,
  RavencoinMainnet: RavencoinMainnet,
  LbryMainnet: LbryMainnet,
  UltraMainnet: UltraMainnet,
  UltraTestnet: UltraTestnet,
  ArbitrumMainnet: ArbitrumMainnet,
  LuxochainMainnet: LuxochainMainnet,
  AlgorandMainnet: AlgorandMainnet,
  FantomMainnet: FantomMainnet,
  OreMainnet: OreMainnet,
  LibreMainnet: LibreMainnet
};
var chainIds = (_chainIds = {}, _defineProperty(_chainIds, Ethereum, (_Ethereum$1 = {}, _defineProperty(_Ethereum$1, EthereumMainnet, '0x005fe7f9'), _defineProperty(_Ethereum$1, EthereumRopsten, '0x0069c322'), _Ethereum$1)), _defineProperty(_chainIds, Arbitrum, _defineProperty({}, ArbitrumMainnet, '0x00ce98c4')), _defineProperty(_chainIds, Luxochain, _defineProperty({}, LuxochainMainnet, '0x00d5beb0')), _defineProperty(_chainIds, BinanceSmartChain, _defineProperty({}, BinanceSmartChainMainnet, '0x00e4b170')), _defineProperty(_chainIds, Algorand, _defineProperty({}, AlgorandMainnet, '0x03c38e67')), _defineProperty(_chainIds, Bitcoin, (_Bitcoin = {}, _defineProperty(_Bitcoin, BitcoinMainnet, '0x01ec97de'), _defineProperty(_Bitcoin, BitcoinTestnet, '0x018afeb2'), _Bitcoin)), _defineProperty(_chainIds, Fantom, _defineProperty({}, FantomMainnet, '0x0022af98')), _defineProperty(_chainIds, Telos, _defineProperty({}, TelosMainnet, '0x028c7109')), _defineProperty(_chainIds, Libre, _defineProperty({}, LibreMainnet, '0x026776fa')), _chainIds);
var pTokens = {
  pBTC: pBTC,
  pLTC: pLTC,
  pETH: pETH,
  pWETH: pWETH,
  pLINK: pLINK,
  pMKR: pMKR,
  pYFI: pYFI,
  PNT: PNT,
  PTERIA: PTERIA,
  pUNI: pUNI,
  pBAND: pBAND,
  pBAL: pBAL,
  pCOMP: pCOMP,
  pSNX: pSNX,
  pOMG: pOMG,
  pDAI: pDAI,
  pANT: pANT,
  pLRC: pLRC,
  pUOS: pUOS,
  pBAT: pBAT,
  pREP: pREP,
  pZRX: pZRX,
  pPNK: pPNK,
  pDOGE: pDOGE,
  pEOS: pEOS,
  IQ: IQ,
  TLOS: TLOS,
  pOPIUM: pOPIUM,
  pBCP: pBCP,
  pDEFIPlusPlus: pDEFIPlusPlus,
  CGG: CGG,
  pUSDT: pUSDT,
  pUSDC: pUSDC,
  pRVN: pRVN,
  pOPEN: pOPEN,
  OCP: OCP,
  ANRX: ANRX,
  TFF: TFF,
  pSAFEMOON: pSAFEMOON,
  EFX: EFX,
  pSEEDS: pSEEDS,
  pLBC: pLBC,
  USDO: USDO,
  GALA: GALA,
  ZMT: ZMT,
  BIST: BIST,
  pVAI: pVAI,
  WSB: WSB,
  LUXO: LUXO,
  pTET: pTET,
  pKEYS: pKEYS,
  OATH: OATH,
  pFTM: pFTM,
  pWFTM: pWFTM,
  NUCO: NUCO,
  ORE: ORE
};

var constants = /*#__PURE__*/Object.freeze({
  __proto__: null,
  blockchains: blockchains,
  networks: networks,
  pTokens: pTokens,
  tokens: tokens,
  chainIds: chainIds
});

function encodeStringForArgs(str) {
  return new Uint8Array(Buffer.from(str));
}
function parseHexString(str) {
  var inStr = str;
  var result = [];
  while (inStr.length >= 2) {
    result.push(parseInt(inStr.substring(0, 2), 16));
    inStr = inStr.substring(2, inStr.length);
  }
  return result;
}
var decodeBlob = function decodeBlob(_blob) {
  return new Uint8Array(Buffer.from(_blob, 'base64').toString('binary').split('').map(function (x) {
    return x.charCodeAt(0);
  }));
};
var redeemFromAlgorand = function () {
  var _ref2 = _asyncToGenerator( _regeneratorRuntime.mark(function _callee(_ref) {
    var amount, to, from, assetIndex, destinationChainId, nativeAccount, client, provider, eventEmitter, swapInfo, suggestedParams, encodedDestinationChainId, asaTransferTx, appCallTx, toBeSignedTxs, signedTxs, binaryAsaTransferSignedTx, binarySignedTxs;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            amount = _ref.amount, to = _ref.to, from = _ref.from, assetIndex = _ref.assetIndex, destinationChainId = _ref.destinationChainId, nativeAccount = _ref.nativeAccount, client = _ref.client, provider = _ref.provider, eventEmitter = _ref.eventEmitter, swapInfo = _ref.swapInfo;
            _context.next = 3;
            return client.getTransactionParams().do();
          case 3:
            suggestedParams = _context.sent;
            encodedDestinationChainId = parseHexString(destinationChainId.substring(2));
            asaTransferTx = algosdk.makeAssetTransferTxnWithSuggestedParamsFromObject({
              from: from,
              to: swapInfo ? algosdk.getApplicationAddress(parseInt(swapInfo.appId)) : to,
              assetIndex: parseInt(swapInfo ? swapInfo.inputAssetId : assetIndex),
              amount: parseInt(amount, 10),
              suggestedParams: suggestedParams,
              note: msgpack.encode([0, encodedDestinationChainId, nativeAccount, []])
            });
            if (swapInfo) {
              appCallTx = algosdk.makeApplicationCallTxnFromObject({
                from: from,
                suggestedParams: suggestedParams,
                appIndex: parseInt(swapInfo.appId),
                appArgs: [encodeStringForArgs('swap'), algosdk.encodeUint64(parseInt(assetIndex, 10)), algosdk.decodeAddress(to).publicKey],
                foreignAssets: [parseInt(assetIndex), parseInt(swapInfo.inputAssetId)],
                accounts: [to]
              });
              algosdk.assignGroupID([asaTransferTx, appCallTx]);
            } else {
              algosdk.assignGroupID([asaTransferTx]);
            }
            toBeSignedTxs = [asaTransferTx];
            if (swapInfo) toBeSignedTxs.push(appCallTx);
            _context.next = 11;
            return provider.signTxn(toBeSignedTxs);
          case 11:
            signedTxs = _context.sent;
            binaryAsaTransferSignedTx = decodeBlob(signedTxs[0].blob ? signedTxs[0].blob : signedTxs[0]);
            binarySignedTxs = [binaryAsaTransferSignedTx];
            if (swapInfo) binarySignedTxs.push(decodeBlob(signedTxs[1].blob ? signedTxs[1].blob : signedTxs[1]));
            _context.next = 17;
            return client.sendRawTransaction(binarySignedTxs).do();
          case 17:
            eventEmitter.emit('hostTxBroadcasted', asaTransferTx.group.toString('base64'));
            _context.next = 20;
            return algosdk.waitForConfirmation(client, asaTransferTx.txID(), 10);
          case 20:
            return _context.abrupt("return", swapInfo ? appCallTx : asaTransferTx);
          case 21:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return function redeemFromAlgorand(_x) {
    return _ref2.apply(this, arguments);
  };
}();

var ____comment = "This file was generated with eosio-abigen. DO NOT EDIT ";
var version = "eosio::abi/1.1";
var types = [
];
var structs = [
	{
		name: "changeissuer",
		base: "",
		fields: [
			{
				name: "issuer",
				type: "name"
			},
			{
				name: "new_issuer",
				type: "name"
			},
			{
				name: "sym",
				type: "symbol"
			}
		]
	},
	{
		name: "close",
		base: "",
		fields: [
			{
				name: "owner",
				type: "name"
			},
			{
				name: "symbol",
				type: "symbol"
			}
		]
	},
	{
		name: "create",
		base: "",
		fields: [
			{
				name: "issuer",
				type: "name"
			},
			{
				name: "maximum_supply",
				type: "asset"
			}
		]
	},
	{
		name: "currency_stats",
		base: "",
		fields: [
			{
				name: "supply",
				type: "asset"
			},
			{
				name: "max_supply",
				type: "asset"
			},
			{
				name: "issuer",
				type: "name"
			}
		]
	},
	{
		name: "issue",
		base: "",
		fields: [
			{
				name: "to",
				type: "name"
			},
			{
				name: "quantity",
				type: "asset"
			},
			{
				name: "memo",
				type: "string"
			}
		]
	},
	{
		name: "open",
		base: "",
		fields: [
			{
				name: "owner",
				type: "name"
			},
			{
				name: "symbol",
				type: "symbol"
			},
			{
				name: "ram_payer",
				type: "name"
			}
		]
	},
	{
		name: "redeem",
		base: "",
		fields: [
			{
				name: "sender",
				type: "name"
			},
			{
				name: "quantity",
				type: "asset"
			},
			{
				name: "memo",
				type: "string"
			}
		]
	},
	{
		name: "retire",
		base: "",
		fields: [
			{
				name: "quantity",
				type: "asset"
			},
			{
				name: "memo",
				type: "string"
			}
		]
	},
	{
		name: "token_account",
		base: "",
		fields: [
			{
				name: "balance",
				type: "asset"
			}
		]
	},
	{
		name: "transfer",
		base: "",
		fields: [
			{
				name: "from",
				type: "name"
			},
			{
				name: "to",
				type: "name"
			},
			{
				name: "quantity",
				type: "asset"
			},
			{
				name: "memo",
				type: "string"
			}
		]
	}
];
var actions = [
	{
		name: "changeissuer",
		type: "changeissuer",
		ricardian_contract: ""
	},
	{
		name: "close",
		type: "close",
		ricardian_contract: ""
	},
	{
		name: "create",
		type: "create",
		ricardian_contract: ""
	},
	{
		name: "issue",
		type: "issue",
		ricardian_contract: ""
	},
	{
		name: "open",
		type: "open",
		ricardian_contract: ""
	},
	{
		name: "redeem",
		type: "redeem",
		ricardian_contract: ""
	},
	{
		name: "retire",
		type: "retire",
		ricardian_contract: ""
	},
	{
		name: "transfer",
		type: "transfer",
		ricardian_contract: ""
	}
];
var tables = [
	{
		name: "accounts",
		type: "token_account",
		index_type: "i64",
		key_names: [
		],
		key_types: [
		]
	},
	{
		name: "stat",
		type: "currency_stats",
		index_type: "i64",
		key_names: [
		],
		key_types: [
		]
	}
];
var ricardian_clauses = [
];
var variants = [
];
var pTokenOnEosAbi = {
	____comment: ____comment,
	version: version,
	types: types,
	structs: structs,
	actions: actions,
	tables: tables,
	ricardian_clauses: ricardian_clauses,
	variants: variants
};

var ____comment$1 = "This file was generated with eosio-abigen. DO NOT EDIT ";
var version$1 = "eosio::abi/1.2";
var types$1 = [
];
var structs$1 = [
	{
		name: "close",
		base: "",
		fields: [
			{
				name: "owner",
				type: "name"
			},
			{
				name: "symbol",
				type: "symbol"
			}
		]
	},
	{
		name: "create",
		base: "",
		fields: [
			{
				name: "issuer",
				type: "name"
			},
			{
				name: "maximum_supply",
				type: "asset"
			}
		]
	},
	{
		name: "currency_stats",
		base: "",
		fields: [
			{
				name: "supply",
				type: "asset"
			},
			{
				name: "max_supply",
				type: "asset"
			},
			{
				name: "issuer",
				type: "name"
			}
		]
	},
	{
		name: "issue",
		base: "",
		fields: [
			{
				name: "to",
				type: "name"
			},
			{
				name: "quantity",
				type: "asset"
			},
			{
				name: "memo",
				type: "string"
			}
		]
	},
	{
		name: "issuewdata",
		base: "",
		fields: [
			{
				name: "to",
				type: "name"
			},
			{
				name: "quantity",
				type: "asset"
			},
			{
				name: "memo",
				type: "string"
			},
			{
				name: "metadata",
				type: "bytes"
			}
		]
	},
	{
		name: "open",
		base: "",
		fields: [
			{
				name: "owner",
				type: "name"
			},
			{
				name: "symbol",
				type: "symbol"
			},
			{
				name: "ram_payer",
				type: "name"
			}
		]
	},
	{
		name: "redeem",
		base: "",
		fields: [
			{
				name: "sender",
				type: "name"
			},
			{
				name: "quantity",
				type: "asset"
			},
			{
				name: "memo",
				type: "string"
			}
		]
	},
	{
		name: "redeem2",
		base: "",
		fields: [
			{
				name: "sender",
				type: "name"
			},
			{
				name: "quantity",
				type: "asset"
			},
			{
				name: "memo",
				type: "string"
			},
			{
				name: "user_data",
				type: "bytes"
			},
			{
				name: "chain_id",
				type: "bytes"
			}
		]
	},
	{
		name: "retire",
		base: "",
		fields: [
			{
				name: "quantity",
				type: "asset"
			},
			{
				name: "memo",
				type: "string"
			}
		]
	},
	{
		name: "token_account",
		base: "",
		fields: [
			{
				name: "balance",
				type: "asset"
			}
		]
	},
	{
		name: "transfer",
		base: "",
		fields: [
			{
				name: "from",
				type: "name"
			},
			{
				name: "to",
				type: "name"
			},
			{
				name: "quantity",
				type: "asset"
			},
			{
				name: "memo",
				type: "string"
			}
		]
	}
];
var actions$1 = [
	{
		name: "close",
		type: "close",
		ricardian_contract: ""
	},
	{
		name: "create",
		type: "create",
		ricardian_contract: ""
	},
	{
		name: "issue",
		type: "issue",
		ricardian_contract: ""
	},
	{
		name: "issuewdata",
		type: "issuewdata",
		ricardian_contract: ""
	},
	{
		name: "open",
		type: "open",
		ricardian_contract: ""
	},
	{
		name: "redeem",
		type: "redeem",
		ricardian_contract: ""
	},
	{
		name: "redeem2",
		type: "redeem2",
		ricardian_contract: ""
	},
	{
		name: "retire",
		type: "retire",
		ricardian_contract: ""
	},
	{
		name: "transfer",
		type: "transfer",
		ricardian_contract: ""
	}
];
var tables$1 = [
	{
		name: "accounts",
		type: "token_account",
		index_type: "i64",
		key_names: [
		],
		key_types: [
		]
	},
	{
		name: "stat",
		type: "currency_stats",
		index_type: "i64",
		key_names: [
		],
		key_types: [
		]
	}
];
var kv_tables = {
};
var ricardian_clauses$1 = [
];
var variants$1 = [
];
var action_results = [
];
var pTokenOnEosAbiV2 = {
	____comment: ____comment$1,
	version: version$1,
	types: types$1,
	structs: structs$1,
	actions: actions$1,
	tables: tables$1,
	kv_tables: kv_tables,
	ricardian_clauses: ricardian_clauses$1,
	variants: variants$1,
	action_results: action_results
};

var redeemFromEosio = function redeemFromEosio(_api, _amount, _nativeAddress, _decimals, _contractAddress, _pToken, _options) {
  try {
    var blocksBehind = _options.blocksBehind,
        expireSeconds = _options.expireSeconds,
        permission = _options.permission,
        actor = _options.actor,
        destinationChainId = _options.destinationChainId,
        _options$version = _options.version,
        version = _options$version === void 0 ? 'v1' : _options$version;
    _api.cachedAbis.set(_contractAddress, {
      abi: version === 'v1' ? pTokenOnEosAbi : pTokenOnEosAbiV2,
      rawAbi: null
    });
    var data = {
      sender: actor,
      quantity: getAmountInEosFormat(_amount, _decimals, _pToken.toUpperCase()),
      memo: _nativeAddress
    };
    if (version === 'v2') {
      data.user_data = '';
      data.chain_id = destinationChainId.substring(2);
    }
    return _api.transact({
      actions: [{
        account: _contractAddress,
        name: version === 'v1' ? 'redeem' : 'redeem2',
        authorization: [{
          actor: actor,
          permission: permission
        }],
        data: data
      }]
    }, {
      blocksBehind: blocksBehind,
      expireSeconds: expireSeconds
    });
  } catch (_err) {
    throw new Error(_err.message);
  }
};

var pTokenOnEthAbi = [
	{
		constant: true,
		inputs: [
		],
		name: "decimals",
		outputs: [
			{
				name: "",
				type: "uint8"
			}
		],
		payable: false,
		stateMutability: "pure",
		type: "function"
	},
	{
		constant: false,
		inputs: [
			{
				name: "amount",
				type: "uint256"
			},
			{
				name: "underlyingAssetRecipient",
				type: "string"
			}
		],
		name: "redeem",
		outputs: [
			{
				name: "",
				type: "bool"
			}
		],
		payable: false,
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		constant: false,
		inputs: [
			{
				name: "amount",
				type: "uint256"
			},
			{
				name: "userData",
				type: "bytes"
			},
			{
				name: "underlyingAssetRecipient",
				type: "string"
			}
		],
		name: "redeem",
		outputs: [
			{
				name: "",
				type: "bool"
			}
		],
		payable: false,
		stateMutability: "nonpayable",
		type: "function"
	}
];

var pTokenOnEthV2 = [
	{
		anonymous: false,
		inputs: [
			{
				indexed: false,
				internalType: "address",
				name: "oldOperator",
				type: "address"
			},
			{
				indexed: false,
				internalType: "address",
				name: "newOperator",
				type: "address"
			}
		],
		name: "AdminOperatorChange",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: false,
				internalType: "address",
				name: "operator",
				type: "address"
			}
		],
		name: "AdminTransferInvoked",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: true,
				internalType: "address",
				name: "owner",
				type: "address"
			},
			{
				indexed: true,
				internalType: "address",
				name: "spender",
				type: "address"
			},
			{
				indexed: false,
				internalType: "uint256",
				name: "value",
				type: "uint256"
			}
		],
		name: "Approval",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: true,
				internalType: "address",
				name: "operator",
				type: "address"
			},
			{
				indexed: true,
				internalType: "address",
				name: "tokenHolder",
				type: "address"
			}
		],
		name: "AuthorizedOperator",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: true,
				internalType: "address",
				name: "operator",
				type: "address"
			},
			{
				indexed: true,
				internalType: "address",
				name: "from",
				type: "address"
			},
			{
				indexed: false,
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			},
			{
				indexed: false,
				internalType: "bytes",
				name: "data",
				type: "bytes"
			},
			{
				indexed: false,
				internalType: "bytes",
				name: "operatorData",
				type: "bytes"
			}
		],
		name: "Burned",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: true,
				internalType: "address",
				name: "operator",
				type: "address"
			},
			{
				indexed: true,
				internalType: "address",
				name: "to",
				type: "address"
			},
			{
				indexed: false,
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			},
			{
				indexed: false,
				internalType: "bytes",
				name: "data",
				type: "bytes"
			},
			{
				indexed: false,
				internalType: "bytes",
				name: "operatorData",
				type: "bytes"
			}
		],
		name: "Minted",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: true,
				internalType: "address",
				name: "previousOwner",
				type: "address"
			},
			{
				indexed: true,
				internalType: "address",
				name: "newOwner",
				type: "address"
			}
		],
		name: "OwnershipTransferred",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: true,
				internalType: "address",
				name: "redeemer",
				type: "address"
			},
			{
				indexed: false,
				internalType: "uint256",
				name: "value",
				type: "uint256"
			},
			{
				indexed: false,
				internalType: "string",
				name: "underlyingAssetRecipient",
				type: "string"
			},
			{
				indexed: false,
				internalType: "bytes",
				name: "userData",
				type: "bytes"
			},
			{
				indexed: false,
				internalType: "bytes4",
				name: "originChainId",
				type: "bytes4"
			},
			{
				indexed: false,
				internalType: "bytes4",
				name: "destinationChainId",
				type: "bytes4"
			}
		],
		name: "Redeem",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: true,
				internalType: "address",
				name: "oldRelayHub",
				type: "address"
			},
			{
				indexed: true,
				internalType: "address",
				name: "newRelayHub",
				type: "address"
			}
		],
		name: "RelayHubChanged",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: true,
				internalType: "address",
				name: "operator",
				type: "address"
			},
			{
				indexed: true,
				internalType: "address",
				name: "tokenHolder",
				type: "address"
			}
		],
		name: "RevokedOperator",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: true,
				internalType: "bytes32",
				name: "role",
				type: "bytes32"
			},
			{
				indexed: true,
				internalType: "bytes32",
				name: "previousAdminRole",
				type: "bytes32"
			},
			{
				indexed: true,
				internalType: "bytes32",
				name: "newAdminRole",
				type: "bytes32"
			}
		],
		name: "RoleAdminChanged",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: true,
				internalType: "bytes32",
				name: "role",
				type: "bytes32"
			},
			{
				indexed: true,
				internalType: "address",
				name: "account",
				type: "address"
			},
			{
				indexed: true,
				internalType: "address",
				name: "sender",
				type: "address"
			}
		],
		name: "RoleGranted",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: true,
				internalType: "bytes32",
				name: "role",
				type: "bytes32"
			},
			{
				indexed: true,
				internalType: "address",
				name: "account",
				type: "address"
			},
			{
				indexed: true,
				internalType: "address",
				name: "sender",
				type: "address"
			}
		],
		name: "RoleRevoked",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: true,
				internalType: "address",
				name: "operator",
				type: "address"
			},
			{
				indexed: true,
				internalType: "address",
				name: "from",
				type: "address"
			},
			{
				indexed: true,
				internalType: "address",
				name: "to",
				type: "address"
			},
			{
				indexed: false,
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			},
			{
				indexed: false,
				internalType: "bytes",
				name: "data",
				type: "bytes"
			},
			{
				indexed: false,
				internalType: "bytes",
				name: "operatorData",
				type: "bytes"
			}
		],
		name: "Sent",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: true,
				internalType: "address",
				name: "from",
				type: "address"
			},
			{
				indexed: true,
				internalType: "address",
				name: "to",
				type: "address"
			},
			{
				indexed: false,
				internalType: "uint256",
				name: "value",
				type: "uint256"
			}
		],
		name: "Transfer",
		type: "event"
	},
	{
		inputs: [
		],
		name: "DEFAULT_ADMIN_ROLE",
		outputs: [
			{
				internalType: "bytes32",
				name: "",
				type: "bytes32"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
		],
		name: "MINTER_ROLE",
		outputs: [
			{
				internalType: "bytes32",
				name: "",
				type: "bytes32"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
		],
		name: "ORIGIN_CHAIN_ID",
		outputs: [
			{
				internalType: "bytes4",
				name: "",
				type: "bytes4"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_gsnTrustedSigner",
				type: "address"
			},
			{
				internalType: "address",
				name: "_gsnFeeTarget",
				type: "address"
			}
		],
		name: "__ERC777GSNUpgradeable_init",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_adminOperator",
				type: "address"
			}
		],
		name: "__ERC777WithAdminOperatorUpgradeable_init",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "relay",
				type: "address"
			},
			{
				internalType: "address",
				name: "from",
				type: "address"
			},
			{
				internalType: "bytes",
				name: "encodedFunction",
				type: "bytes"
			},
			{
				internalType: "uint256",
				name: "transactionFee",
				type: "uint256"
			},
			{
				internalType: "uint256",
				name: "gasPrice",
				type: "uint256"
			},
			{
				internalType: "uint256",
				name: "gasLimit",
				type: "uint256"
			},
			{
				internalType: "uint256",
				name: "nonce",
				type: "uint256"
			},
			{
				internalType: "bytes",
				name: "approvalData",
				type: "bytes"
			},
			{
				internalType: "uint256",
				name: "",
				type: "uint256"
			}
		],
		name: "acceptRelayedCall",
		outputs: [
			{
				internalType: "uint256",
				name: "",
				type: "uint256"
			},
			{
				internalType: "bytes",
				name: "",
				type: "bytes"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
		],
		name: "adminOperator",
		outputs: [
			{
				internalType: "address",
				name: "",
				type: "address"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "sender",
				type: "address"
			},
			{
				internalType: "address",
				name: "recipient",
				type: "address"
			},
			{
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			},
			{
				internalType: "bytes",
				name: "data",
				type: "bytes"
			},
			{
				internalType: "bytes",
				name: "operatorData",
				type: "bytes"
			}
		],
		name: "adminTransfer",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "holder",
				type: "address"
			},
			{
				internalType: "address",
				name: "spender",
				type: "address"
			}
		],
		name: "allowance",
		outputs: [
			{
				internalType: "uint256",
				name: "",
				type: "uint256"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "spender",
				type: "address"
			},
			{
				internalType: "uint256",
				name: "value",
				type: "uint256"
			}
		],
		name: "approve",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "operator",
				type: "address"
			}
		],
		name: "authorizeOperator",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "tokenHolder",
				type: "address"
			}
		],
		name: "balanceOf",
		outputs: [
			{
				internalType: "uint256",
				name: "",
				type: "uint256"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			},
			{
				internalType: "bytes",
				name: "data",
				type: "bytes"
			}
		],
		name: "burn",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "bytes4",
				name: "_newOriginChainId",
				type: "bytes4"
			}
		],
		name: "changeOriginChainId",
		outputs: [
			{
				internalType: "bool",
				name: "success",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
		],
		name: "decimals",
		outputs: [
			{
				internalType: "uint8",
				name: "",
				type: "uint8"
			}
		],
		stateMutability: "pure",
		type: "function"
	},
	{
		inputs: [
		],
		name: "defaultOperators",
		outputs: [
			{
				internalType: "address[]",
				name: "",
				type: "address[]"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
		],
		name: "getHubAddr",
		outputs: [
			{
				internalType: "address",
				name: "",
				type: "address"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "bytes32",
				name: "role",
				type: "bytes32"
			}
		],
		name: "getRoleAdmin",
		outputs: [
			{
				internalType: "bytes32",
				name: "",
				type: "bytes32"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "bytes32",
				name: "role",
				type: "bytes32"
			},
			{
				internalType: "uint256",
				name: "index",
				type: "uint256"
			}
		],
		name: "getRoleMember",
		outputs: [
			{
				internalType: "address",
				name: "",
				type: "address"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "bytes32",
				name: "role",
				type: "bytes32"
			}
		],
		name: "getRoleMemberCount",
		outputs: [
			{
				internalType: "uint256",
				name: "",
				type: "uint256"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_account",
				type: "address"
			}
		],
		name: "grantMinterRole",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "bytes32",
				name: "role",
				type: "bytes32"
			},
			{
				internalType: "address",
				name: "account",
				type: "address"
			}
		],
		name: "grantRole",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
		],
		name: "granularity",
		outputs: [
			{
				internalType: "uint256",
				name: "",
				type: "uint256"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
		],
		name: "gsnExtraGas",
		outputs: [
			{
				internalType: "uint256",
				name: "",
				type: "uint256"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
		],
		name: "gsnFeeTarget",
		outputs: [
			{
				internalType: "address",
				name: "",
				type: "address"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
		],
		name: "gsnTrustedSigner",
		outputs: [
			{
				internalType: "address",
				name: "",
				type: "address"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_account",
				type: "address"
			}
		],
		name: "hasMinterRole",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "bytes32",
				name: "role",
				type: "bytes32"
			},
			{
				internalType: "address",
				name: "account",
				type: "address"
			}
		],
		name: "hasRole",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "string",
				name: "tokenName",
				type: "string"
			},
			{
				internalType: "string",
				name: "tokenSymbol",
				type: "string"
			},
			{
				internalType: "address",
				name: "defaultAdmin",
				type: "address"
			},
			{
				internalType: "bytes4",
				name: "originChainId",
				type: "bytes4"
			}
		],
		name: "initialize",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "operator",
				type: "address"
			},
			{
				internalType: "address",
				name: "tokenHolder",
				type: "address"
			}
		],
		name: "isOperatorFor",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "recipient",
				type: "address"
			},
			{
				internalType: "uint256",
				name: "value",
				type: "uint256"
			}
		],
		name: "mint",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "recipient",
				type: "address"
			},
			{
				internalType: "uint256",
				name: "value",
				type: "uint256"
			},
			{
				internalType: "bytes",
				name: "userData",
				type: "bytes"
			},
			{
				internalType: "bytes",
				name: "operatorData",
				type: "bytes"
			}
		],
		name: "mint",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
		],
		name: "name",
		outputs: [
			{
				internalType: "string",
				name: "",
				type: "string"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "account",
				type: "address"
			},
			{
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			},
			{
				internalType: "bytes",
				name: "data",
				type: "bytes"
			},
			{
				internalType: "bytes",
				name: "operatorData",
				type: "bytes"
			}
		],
		name: "operatorBurn",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "account",
				type: "address"
			},
			{
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			},
			{
				internalType: "bytes",
				name: "userData",
				type: "bytes"
			},
			{
				internalType: "bytes",
				name: "operatorData",
				type: "bytes"
			},
			{
				internalType: "string",
				name: "underlyingAssetRecipient",
				type: "string"
			},
			{
				internalType: "bytes4",
				name: "destinationChainId",
				type: "bytes4"
			}
		],
		name: "operatorRedeem",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "sender",
				type: "address"
			},
			{
				internalType: "address",
				name: "recipient",
				type: "address"
			},
			{
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			},
			{
				internalType: "bytes",
				name: "data",
				type: "bytes"
			},
			{
				internalType: "bytes",
				name: "operatorData",
				type: "bytes"
			}
		],
		name: "operatorSend",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
		],
		name: "owner",
		outputs: [
			{
				internalType: "address",
				name: "",
				type: "address"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "bytes",
				name: "context",
				type: "bytes"
			},
			{
				internalType: "bool",
				name: "success",
				type: "bool"
			},
			{
				internalType: "uint256",
				name: "actualCharge",
				type: "uint256"
			},
			{
				internalType: "bytes32",
				name: "preRetVal",
				type: "bytes32"
			}
		],
		name: "postRelayedCall",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "bytes",
				name: "context",
				type: "bytes"
			}
		],
		name: "preRelayedCall",
		outputs: [
			{
				internalType: "bytes32",
				name: "",
				type: "bytes32"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			},
			{
				internalType: "bytes",
				name: "userData",
				type: "bytes"
			},
			{
				internalType: "string",
				name: "underlyingAssetRecipient",
				type: "string"
			},
			{
				internalType: "bytes4",
				name: "destinationChainId",
				type: "bytes4"
			}
		],
		name: "redeem",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			},
			{
				internalType: "string",
				name: "underlyingAssetRecipient",
				type: "string"
			},
			{
				internalType: "bytes4",
				name: "destinationChainId",
				type: "bytes4"
			}
		],
		name: "redeem",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
		],
		name: "relayHubVersion",
		outputs: [
			{
				internalType: "string",
				name: "",
				type: "string"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
		],
		name: "renounceOwnership",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "bytes32",
				name: "role",
				type: "bytes32"
			},
			{
				internalType: "address",
				name: "account",
				type: "address"
			}
		],
		name: "renounceRole",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_account",
				type: "address"
			}
		],
		name: "revokeMinterRole",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "operator",
				type: "address"
			}
		],
		name: "revokeOperator",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "bytes32",
				name: "role",
				type: "bytes32"
			},
			{
				internalType: "address",
				name: "account",
				type: "address"
			}
		],
		name: "revokeRole",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "recipient",
				type: "address"
			},
			{
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			},
			{
				internalType: "bytes",
				name: "data",
				type: "bytes"
			}
		],
		name: "send",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "adminOperator_",
				type: "address"
			}
		],
		name: "setAdminOperator",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_gsnFeeTarget",
				type: "address"
			}
		],
		name: "setFeeTarget",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "uint256",
				name: "_gsnExtraGas",
				type: "uint256"
			}
		],
		name: "setGSNExtraGas",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_gsnTrustedSigner",
				type: "address"
			}
		],
		name: "setTrustedSigner",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
		],
		name: "symbol",
		outputs: [
			{
				internalType: "string",
				name: "",
				type: "string"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
		],
		name: "totalSupply",
		outputs: [
			{
				internalType: "uint256",
				name: "",
				type: "uint256"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "recipient",
				type: "address"
			},
			{
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			}
		],
		name: "transfer",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "holder",
				type: "address"
			},
			{
				internalType: "address",
				name: "recipient",
				type: "address"
			},
			{
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			}
		],
		name: "transferFrom",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "newOwner",
				type: "address"
			}
		],
		name: "transferOwnership",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	}
];

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
var redeemFromEvmCompatible = function redeemFromEvmCompatible(_web3, _options, _params, _promiEvent, _broadcastEventName) {
  var _version = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 'v1';
  return new Promise(function (_resolve, _reject) {
    eth[_options.privateKey ? 'sendSignedMethodTx' : 'makeContractSend'](_web3, 'redeem', _objectSpread({
      abi: _version === 'v1' ? pTokenOnEthAbi : pTokenOnEthV2
    }, _options), _params).once('transactionHash', function (_hash) {
      _promiEvent.eventEmitter.emit('onEthTxBroadcasted', _hash);
      _promiEvent.eventEmitter.emit(_broadcastEventName, _hash);
    }).once('receipt', _resolve).once('error', _reject);
  });
};

var pERC20VaultContractAbi = [
	{
		inputs: [
			{
				internalType: "address",
				name: "_weth",
				type: "address"
			},
			{
				internalType: "address[]",
				name: "_tokensToSupport",
				type: "address[]"
			}
		],
		stateMutability: "nonpayable",
		type: "constructor"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: true,
				internalType: "address",
				name: "_from",
				type: "address"
			},
			{
				indexed: true,
				internalType: "address",
				name: "_token",
				type: "address"
			},
			{
				indexed: false,
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			}
		],
		name: "LogWithdrawToken",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: false,
				internalType: "address",
				name: "_tokenAddress",
				type: "address"
			},
			{
				indexed: false,
				internalType: "address",
				name: "_tokenSender",
				type: "address"
			},
			{
				indexed: false,
				internalType: "uint256",
				name: "_tokenAmount",
				type: "uint256"
			},
			{
				indexed: false,
				internalType: "string",
				name: "_destinationAddress",
				type: "string"
			}
		],
		name: "PegIn",
		type: "event"
	},
	{
		inputs: [
		],
		name: "PNETWORK",
		outputs: [
			{
				internalType: "address",
				name: "",
				type: "address"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "asset",
				type: "address"
			}
		],
		name: "adminWithdraw",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
		],
		name: "weth",
		outputs: [
			{
				internalType: "contract IWETH",
				name: "",
				type: "address"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		stateMutability: "payable",
		type: "receive"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_weth",
				type: "address"
			}
		],
		name: "setWeth",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_token",
				type: "address"
			}
		],
		name: "IS_TOKEN_SUPPORTED",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_tokenAddress",
				type: "address"
			}
		],
		name: "addSupportedToken",
		outputs: [
			{
				internalType: "bool",
				name: "SUCCESS",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_tokenAddress",
				type: "address"
			}
		],
		name: "removeSupportedToken",
		outputs: [
			{
				internalType: "bool",
				name: "SUCCESS",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "uint256",
				name: "_tokenAmount",
				type: "uint256"
			},
			{
				internalType: "address",
				name: "_tokenAddress",
				type: "address"
			},
			{
				internalType: "string",
				name: "_destinationAddress",
				type: "string"
			}
		],
		name: "pegIn",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "uint256",
				name: "_tokenAmount",
				type: "uint256"
			},
			{
				internalType: "address",
				name: "_tokenAddress",
				type: "address"
			},
			{
				internalType: "string",
				name: "_destinationAddress",
				type: "string"
			},
			{
				internalType: "bytes",
				name: "_userData",
				type: "bytes"
			}
		],
		name: "pegIn",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "",
				type: "address"
			},
			{
				internalType: "address",
				name: "from",
				type: "address"
			},
			{
				internalType: "address",
				name: "to",
				type: "address"
			},
			{
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			},
			{
				internalType: "bytes",
				name: "userData",
				type: "bytes"
			},
			{
				internalType: "bytes",
				name: "",
				type: "bytes"
			}
		],
		name: "tokensReceived",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "string",
				name: "_destinationAddress",
				type: "string"
			}
		],
		name: "pegInEth",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "payable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "string",
				name: "_destinationAddress",
				type: "string"
			},
			{
				internalType: "bytes",
				name: "_userData",
				type: "bytes"
			}
		],
		name: "pegInEth",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "payable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address payable",
				name: "_tokenRecipient",
				type: "address"
			},
			{
				internalType: "address",
				name: "_tokenAddress",
				type: "address"
			},
			{
				internalType: "uint256",
				name: "_tokenAmount",
				type: "uint256"
			}
		],
		name: "pegOut",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address payable",
				name: "_to",
				type: "address"
			}
		],
		name: "migrate",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
		],
		name: "destroy",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address payable",
				name: "_to",
				type: "address"
			},
			{
				internalType: "address",
				name: "_tokenAddress",
				type: "address"
			}
		],
		name: "migrateSingle",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	}
];

var pERC20VaultV2ContractAbi = [
	{
		anonymous: false,
		inputs: [
			{
				indexed: true,
				internalType: "address",
				name: "_from",
				type: "address"
			},
			{
				indexed: true,
				internalType: "address",
				name: "_token",
				type: "address"
			},
			{
				indexed: false,
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			}
		],
		name: "LogWithdrawToken",
		type: "event"
	},
	{
		anonymous: false,
		inputs: [
			{
				indexed: false,
				internalType: "address",
				name: "_tokenAddress",
				type: "address"
			},
			{
				indexed: false,
				internalType: "address",
				name: "_tokenSender",
				type: "address"
			},
			{
				indexed: false,
				internalType: "uint256",
				name: "_tokenAmount",
				type: "uint256"
			},
			{
				indexed: false,
				internalType: "string",
				name: "_destinationAddress",
				type: "string"
			},
			{
				indexed: false,
				internalType: "bytes",
				name: "_userData",
				type: "bytes"
			},
			{
				indexed: false,
				internalType: "bytes4",
				name: "_originChainId",
				type: "bytes4"
			},
			{
				indexed: false,
				internalType: "bytes4",
				name: "_destinationChainId",
				type: "bytes4"
			}
		],
		name: "PegIn",
		type: "event"
	},
	{
		inputs: [
		],
		name: "ORIGIN_CHAIN_ID",
		outputs: [
			{
				internalType: "bytes4",
				name: "",
				type: "bytes4"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
		],
		name: "PNETWORK",
		outputs: [
			{
				internalType: "address",
				name: "",
				type: "address"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_tokenAddress",
				type: "address"
			}
		],
		name: "addSupportedToken",
		outputs: [
			{
				internalType: "bool",
				name: "SUCCESS",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "asset",
				type: "address"
			}
		],
		name: "adminWithdraw",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "bytes4",
				name: "_newOriginChainId",
				type: "bytes4"
			}
		],
		name: "changeOriginChainId",
		outputs: [
			{
				internalType: "bool",
				name: "success",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
		],
		name: "getSupportedTokens",
		outputs: [
			{
				internalType: "address[]",
				name: "res",
				type: "address[]"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_weth",
				type: "address"
			},
			{
				internalType: "address[]",
				name: "_tokensToSupport",
				type: "address[]"
			},
			{
				internalType: "bytes4",
				name: "_originChainId",
				type: "bytes4"
			}
		],
		name: "initialize",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_token",
				type: "address"
			}
		],
		name: "isTokenSupported",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "uint256",
				name: "_tokenAmount",
				type: "uint256"
			},
			{
				internalType: "address",
				name: "_tokenAddress",
				type: "address"
			},
			{
				internalType: "string",
				name: "_destinationAddress",
				type: "string"
			},
			{
				internalType: "bytes4",
				name: "_destinationChainId",
				type: "bytes4"
			}
		],
		name: "pegIn",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "uint256",
				name: "_tokenAmount",
				type: "uint256"
			},
			{
				internalType: "address",
				name: "_tokenAddress",
				type: "address"
			},
			{
				internalType: "string",
				name: "_destinationAddress",
				type: "string"
			},
			{
				internalType: "bytes",
				name: "_userData",
				type: "bytes"
			},
			{
				internalType: "bytes4",
				name: "_destinationChainId",
				type: "bytes4"
			}
		],
		name: "pegIn",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "string",
				name: "_destinationAddress",
				type: "string"
			},
			{
				internalType: "bytes4",
				name: "_destinationChainId",
				type: "bytes4"
			},
			{
				internalType: "bytes",
				name: "_userData",
				type: "bytes"
			}
		],
		name: "pegInEth",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "payable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "string",
				name: "_destinationAddress",
				type: "string"
			},
			{
				internalType: "bytes4",
				name: "_destinationChainId",
				type: "bytes4"
			}
		],
		name: "pegInEth",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "payable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address payable",
				name: "_tokenRecipient",
				type: "address"
			},
			{
				internalType: "address",
				name: "_tokenAddress",
				type: "address"
			},
			{
				internalType: "uint256",
				name: "_tokenAmount",
				type: "uint256"
			},
			{
				internalType: "bytes",
				name: "_userData",
				type: "bytes"
			}
		],
		name: "pegOut",
		outputs: [
			{
				internalType: "bool",
				name: "success",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address payable",
				name: "_tokenRecipient",
				type: "address"
			},
			{
				internalType: "address",
				name: "_tokenAddress",
				type: "address"
			},
			{
				internalType: "uint256",
				name: "_tokenAmount",
				type: "uint256"
			}
		],
		name: "pegOut",
		outputs: [
			{
				internalType: "bool",
				name: "",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_tokenAddress",
				type: "address"
			}
		],
		name: "removeSupportedToken",
		outputs: [
			{
				internalType: "bool",
				name: "SUCCESS",
				type: "bool"
			}
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_pnetwork",
				type: "address"
			}
		],
		name: "setPNetwork",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "_weth",
				type: "address"
			}
		],
		name: "setWeth",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
			{
				internalType: "address",
				name: "",
				type: "address"
			},
			{
				internalType: "address",
				name: "from",
				type: "address"
			},
			{
				internalType: "address",
				name: "to",
				type: "address"
			},
			{
				internalType: "uint256",
				name: "amount",
				type: "uint256"
			},
			{
				internalType: "bytes",
				name: "userData",
				type: "bytes"
			},
			{
				internalType: "bytes",
				name: "",
				type: "bytes"
			}
		],
		name: "tokensReceived",
		outputs: [
		],
		stateMutability: "nonpayable",
		type: "function"
	},
	{
		inputs: [
		],
		name: "weth",
		outputs: [
			{
				internalType: "contract IWETH",
				name: "",
				type: "address"
			}
		],
		stateMutability: "view",
		type: "function"
	},
	{
		stateMutability: "payable",
		type: "receive"
	}
];

var version$2 = "eosio::abi/1.1";
var types$2 = [
];
var structs$2 = [
	{
		name: "create",
		base: "",
		fields: [
			{
				name: "issuer",
				type: "name"
			},
			{
				name: "maximum_supply",
				type: "asset"
			}
		]
	},
	{
		name: "issue",
		base: "",
		fields: [
			{
				name: "to",
				type: "name"
			},
			{
				name: "quantity",
				type: "asset"
			},
			{
				name: "memo",
				type: "string"
			}
		]
	},
	{
		name: "retire",
		base: "",
		fields: [
			{
				name: "quantity",
				type: "asset"
			},
			{
				name: "memo",
				type: "string"
			}
		]
	},
	{
		name: "close",
		base: "",
		fields: [
			{
				name: "owner",
				type: "name"
			},
			{
				name: "symbol",
				type: "symbol"
			}
		]
	},
	{
		name: "transfer",
		base: "",
		fields: [
			{
				name: "from",
				type: "name"
			},
			{
				name: "to",
				type: "name"
			},
			{
				name: "quantity",
				type: "asset"
			},
			{
				name: "memo",
				type: "string"
			}
		]
	},
	{
		name: "account",
		base: "",
		fields: [
			{
				name: "balance",
				type: "asset"
			}
		]
	},
	{
		name: "currency_stats",
		base: "",
		fields: [
			{
				name: "supply",
				type: "asset"
			},
			{
				name: "max_supply",
				type: "asset"
			},
			{
				name: "issuer",
				type: "name"
			}
		]
	}
];
var actions$2 = [
	{
		name: "transfer",
		type: "transfer",
		ricardian_contract: ""
	},
	{
		name: "issue",
		type: "issue",
		ricardian_contract: ""
	},
	{
		name: "retire",
		type: "retire",
		ricardian_contract: ""
	},
	{
		name: "create",
		type: "create",
		ricardian_contract: ""
	},
	{
		name: "close",
		type: "close",
		ricardian_contract: ""
	}
];
var tables$2 = [
	{
		name: "accounts",
		type: "account",
		index_type: "i64",
		key_names: [
			"currency"
		],
		key_types: [
			"uint64"
		]
	},
	{
		name: "stat",
		type: "currency_stats",
		index_type: "i64",
		key_names: [
			"currency"
		],
		key_types: [
			"uint64"
		]
	}
];
var ricardian_clauses$2 = [
];
var abi_extensions = [
];
var EosioTokenAbi = {
	version: version$2,
	types: types$2,
	structs: structs$2,
	actions: actions$2,
	tables: tables$2,
	ricardian_clauses: ricardian_clauses$2,
	abi_extensions: abi_extensions
};

var index = {
  abi: {
    pTokenOnEth: pTokenOnEthAbi,
    pTokenOnEos: pTokenOnEosAbi,
    pERC20Vault: pERC20VaultContractAbi,
    pERC20VaultV2: pERC20VaultV2ContractAbi,
    EosioToken: EosioTokenAbi
  },
  algo: algo,
  btc: btc,
  doge: doge,
  converters: converters,
  constants: constants,
  eth: eth,
  eos: eos,
  telos: eos,
  libre: eos,
  helpers: helpers,
  ltc: ltc,
  rvn: rvn,
  redeemFrom: {
    redeemFromEvmCompatible: redeemFromEvmCompatible,
    redeemFromEosio: redeemFromEosio,
    redeemFromAlgorand: redeemFromAlgorand
  }
};

module.exports = index;
